const YAML = require('yamljs');
const fs = require('fs');
const _ = require('lodash');
const joiToSwagger = require('joi-to-swagger');
const fse = require('fs-extra');

const apiYaml  = fs.readFileSync("gen/openapi.yaml", "utf8");
const pathsYaml = fs.readFileSync("gen/operations.yaml", "utf8");

const api = YAML.parse(apiYaml);
const operations = YAML.parse(pathsYaml);

api.tags = operations.map(({name, description}) => ({
    name,
    description
}));

process.env.apiGen = true;

const generatePaths = (operations) => {
    const out = {};
    operations.map(operation => {
        const operationApi = require(`./../../backend/src/${operation.file}`);
        const routes = operationApi.handler[operation.routes].filter(route =>
            !((route.scopes && route.scopes.includes("occm-access")) || (route.scopes && route.scopes.includes("occm-create")) || (route.doc && route.doc.hidden))
        );
        routes.map(route => {
            const identifier = `API ${route.method} ${route.pattern}`;

            if(!route.doc) throw new Error(`${identifier} is missing doc object`);

            const path = out[route.pattern] = out[route.pattern] || {};
            const method = path[route.method.toLowerCase()] = {};


            method.tags = [operation.name];


            const responses = method.responses = {};
            responses[401] = {"$ref": "#/components/responses/401"};
            responses[403] = {"$ref": "#/components/responses/403"};

            if(route.doc.responses){
                Object.assign(responses, route.doc.responses);
                _.forEach(route.doc.responses, (val, key) => {
                    if(_.isString(val) && val !== "#ref"){
                        const docYaml = fs.readFileSync(`gen/docs/${val}`, "utf8");
                        val = YAML.parse(docYaml);
                        responses[key] = val;
                    }
                    if(val !== "#ref" && !_.isObject(val) && !val.description) throw new Error(`${identifier} response ${key} does not have a description`);
                });

                if(responses[204]){
                    responses[204] = {"$ref": "#/components/responses/204"};
                }
            }


            if(!route.doc.summary) throw new Error(`${identifier} is missing doc summary`);
            method.summary = route.doc.summary;


            if(route.bodySchema){
                const requestBody = method.requestBody = {};
                requestBody.required = !route.bodyOptional;

                const swagger = joiToSwagger(route.bodySchema);

                if(swagger.swagger.required){
                    swagger.swagger.required.forEach(requiredProperty => {
                       swagger.swagger.properties[requiredProperty].required = true;
                    });
                }

                requestBody.content = {
                    "application/json": {
                        "schema": {
                            type: "object",
                            properties: swagger.swagger.properties
                        }
                    }
                }
            }
        });
    });

    return out;
};

api.paths = generatePaths(operations);

console.log("Removing previous build directory");
fse.removeSync("build-gen");

console.log("Create new build directory");
fs.mkdirSync("build-gen");

api.servers = [{url: process.env.REACT_APP_PORTAL_BACKEND_BASE_URL}];

console.log("writing json and yaml files");
fs.writeFileSync("build-gen/openapi.json", JSON.stringify(api));
fs.writeFileSync("build-gen/openapi.yaml", YAML.stringify(api));
