const Joi = require('joi');
const _ = require("lodash");
const AWS = require('aws-sdk');

const { authAxios } = require("./auth-utils");
const dispatch = require('./dispatch');
const clientManagementHandler = require("./management/clientManagementHandler");
const { resetSubordinatePassword } = require("./management/courseManagementHandler");
const { enableConnection, disableConnection, getAllConnectionsWithRulesInfo } = require('./utils/connection');

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const {
    getUsersWrapper, getUserBlocks,
    unblockUser, generatePasswordChangeTicket, getUserLogs, resetMfa
} = require("./users-governance");

const deleteOccms = ({ occms, userId }) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    return authAxios().then(client => {
        const requestParams = {
            RequestItems: {
                [process.env.occmClientInfoTable]: {
                    Keys: _(occms)
                        .take(100) // batchGet is limited to 100
                        .map(clientId => ({ clientId }))
                        .value()
                }
            }
        };
        if (occms.length === 0) {
            return false;
        }
        return dynamoDb.batchGet(requestParams).promise().then(batchGetResponse => {
            const responseArr = _.get(batchGetResponse, `Responses.${process.env.occmClientInfoTable}`, []);
            return Promise.all(responseArr.map(resp => {
                if (resp.setupUserId === userId) { // check if authorized to delete client
                    const clientParams = {
                        TableName: process.env.occmClientInfoTable,
                        Key: { clientId: resp.clientId }
                    };
                    return dynamoDb.delete(clientParams).promise()// delete from dynamo
                        .then(() => client.delete(`/clients/${resp.clientId}`)) // delete from auth0
                        .catch(err => {
                            console.log(`Failed to delete client with Id: ${resp.clientId}`);
                            console.log(err);
                        });
                }

                console.log(`Not deleting client ${resp.clientId}. Setup user ${resp.setupUserId} does not match`);
                return null;
            })).then(() => true);
        });
    }).then(hadOccms => {
        if (hadOccms) { console.log("Deleted occm clients"); }
        return true;
    }).catch(err => {
        console.log("Failed to delete occm clients and dynamo data");
        console.log(err);
    });
};

const deleteDynamoEntriesForUser = exports.deleteDynamoEntriesForUser = ({ userId }) => {
    if (process.env.isTest) { return Promise.resolve(); }
    console.log(`Deleting dynamo entries for user ${userId}`);
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: {
            userId: `${userId}`
        }
    };

    const deleteUserInfoP = dynamoDb.delete(userInfoParams).promise().catch(err => {
        console.log("Failed to delete user info entry");
        console.log(err);
    });

    const userPromoParams = {
        TableName: process.env.userPromoTable,
        KeyConditionExpression: 'userId = :userId',
        ExpressionAttributeValues: {
            ':userId': userId.toString()
        }
    };

    const userPromosP = dynamoDb.query(userPromoParams).promise().then(({ Items: userPromos }) => Promise.all(userPromos.map(userPromo => {
        const deleteParams = {
            TableName: process.env.userPromoTable,
            Key: {
                userId: userPromo.userId,
                promoId: userPromo.promoId
            }
        };

        return dynamoDb.delete(deleteParams).promise().catch(err => {
            console.log("Failed to delete userPromo entry");
            console.log(err);
        });
    }))).catch(err => {
        console.log("Failed to delete userPromos entries");
        console.log(err);
    });

    const userNotificationsParams = {
        TableName: process.env.notificationsTable,
        KeyConditionExpression: 'userId = :userId',
        ExpressionAttributeValues: {
            ':userId': userId.toString()
        }
    };

    const userNotificationsP = dynamoDb.query(userNotificationsParams).promise().then(({ Items: userNotifications }) => Promise.all(userNotifications.map(notification => {
        const deleteParams = {
            TableName: process.env.notificationsTable,
            Key: {
                userId: notification.userId,
                createDate: notification.createDate
            }
        };

        return dynamoDb.delete(deleteParams).promise().catch(err => {
            console.log("Failed to delete user notifications");
            console.log(err);
        });
    }))).catch(err => {
        console.log("Failed to delete user notifications");
        console.log(err);
    });

    return Promise.all([deleteUserInfoP, userPromosP, userNotificationsP]).catch(err => {
        console.log("Failed to clear dynamo entries for user");
        console.log(err);
    });
};

const deleteUser = exports.deleteUser = ({ userId, occms }) => {
    console.log(`Attempting to delete user with id: ${userId} and associated occms: ${occms}`);
    return deleteOccms({ occms: occms || [], userId }).then(() => authAxios().then(client => client.delete(`users/${userId}`).then(({ data }) => {
        console.log(data);
    })
        .then(() => deleteDynamoEntriesForUser({ userId })))).then(() => {
        console.log(`User deletion successful`);
        return null;
    }, err => {
        console.error(`User deletion failed`, err);
        throw err;
    });
};

const deleteOccmClientsCreatedBy = exports.deleteOccmClientsCreatedBy = ({ userId }) => {
    console.log(`Attempting to delete all occm clients created by ${userId}`);
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: { userId }
    };

    return dynamoDb.get(userInfoParams).promise().then(userInfoResponse => {
        const userInfo = userInfoResponse.Item || {};
        const occms = _.get(userInfo, "associatedOccms.values", []);

        if (!occms.length) {
            console.log('No associated occms to delete');
            return false;
        }

        return deleteOccms({ occms, userId }).then(() => {
            console.log("Removing occm clientId's from associated occms list");
            const dynamoDbOld = new AWS.DynamoDB();
            const params = {
                TableName: process.env.userInfoTable,
                ExpressionAttributeValues: {
                    ":c": { SS: occms }
                },
                Key: {
                    userId: {
                        S: userId
                    }
                },
                UpdateExpression: "DELETE associatedOccms :c"
            };

            return dynamoDbOld.updateItem(params).promise().then(() => true);
        });
    }).then(hadOccms => {
        if (hadOccms) { console.log("Occm clients deleted and associations removed"); }
        return null;
    })
        .catch(err => {
            console.log("Error deleting clients");
            throw err;
        });
};

const routes = [
    {
        method: "POST",
        pattern: "/governance/delete-user",
        bodySchema: Joi.object().keys({
            occms: Joi.array()
        }),
        bodyOptional: true,
        handler: ({ principalId, body }) => deleteUser({ userId: principalId, occms: _.get(body, "occms", null) })
    },
    {
        method: "POST",
        pattern: "/governance/delete-associated-occms",
        handler: ({ principalId }) => deleteOccmClientsCreatedBy({ userId: principalId })
    },
    {
        method: "POST",
        pattern: "/governance/enable-connection",
        bodySchema: Joi.object().keys({
            connectionName: Joi.string(),
            companyName: Joi.string().min(3),
            emailDomains: Joi.string()
        }),
        scopes: ['cc:support'],
        handler: ({ body: { connectionName, companyName, emailDomains } }) => enableConnection({ connectionName, companyName, emailDomains })
    },
    {
        method: "POST",
        pattern: "/governance/disable-connection",
        bodySchema: Joi.object().keys({
            connectionName: Joi.string()
        }),
        scopes: ['cc:support'],
        handler: ({ body: { connectionName } }) => disableConnection({ connectionName })
    },
    {
        method: "GET",
        pattern: "/governance/connections",
        scopes: ['cc:support'],
        handler: ({ queryParams }) => getAllConnectionsWithRulesInfo({ fields: queryParams ? queryParams.fields : null })
    },
    {
        method: "POST",
        pattern: "/governance/course/reset-password",
        bodySchema: Joi.object().keys({
            subordinateEmail: Joi.string().required().email(),
            subordinateNewPassword: Joi.string().required()
        }),
        scopes: ['cc:course-admin'],
        handler: ({ principalId, body: { subordinateEmail, subordinateNewPassword } }) =>
            resetSubordinatePassword({ userId: principalId, subordinateEmail, subordinateNewPassword })
    }
];

const supportRoutes = [
    {
        method: "GET",
        pattern: "/governance/support/users",
        querySchema: Joi.object().keys({
            fields: Joi.string().allow(''),
            q: Joi.string().required()
        }),
        scopes: ['cc:support'],
        handler: ({ queryParams: { fields, q } }) => getUsersWrapper({ fields, q })
    },
    {
        method: "GET",
        pattern: "/governance/support/users/:userId/blocks",
        scopes: ['cc:support'],
        handler: ({ urlParams }) => getUserBlocks({ userId: urlParams.userId })
    },
    {
        method: "DELETE",
        pattern: "/governance/support/users/:userId/blocks",
        scopes: ['cc:support'],
        handler: ({ urlParams }) => unblockUser({ userId: urlParams.userId })
    },
    {
        method: "POST",
        pattern: "/governance/support/users/:userId/password-change-ticket",
        scopes: ['cc:support'],
        handler: ({ urlParams }) => generatePasswordChangeTicket({ userId: urlParams.userId })
    },
    {
        method: "GET",
        pattern: "/governance/support/users/:userId/logs",
        querySchema: Joi.object().keys({
            page: Joi.number().integer().min(0).options({ convert: true }),
            perPage: Joi.number().integer().min(0).options({ convert: true })
        }),
        scopes: ['cc:support'],
        handler: ({ urlParams, queryParams: { page, perPage } }) => getUserLogs({ userId: urlParams.userId, page, perPage })
    },
    {
        method: "POST",
        pattern: "/governance/support/users/:userId/reset-mfa",
        scopes: ['cc:support'],
        handler: ({ urlParams }) => resetMfa({ userId: urlParams.userId })
    }
];

const developersRoutes = [
    {
        method: "GET",
        pattern: "/governance/developers/clients",
        scopes: ["cc:clients"],
        handler: ({ principalId }) => clientManagementHandler.getClients({ userId: principalId })
    },
    {
        method: "GET",
        pattern: "/governance/developers/clients/:clientId",
        scopes: ["cc:clients"],
        handler: ({ urlParams, principalId }) => clientManagementHandler.getClient({ clientId: urlParams.clientId, userId: principalId })
    },
    {
        method: "PATCH",
        pattern: "/governance/developers/clients/:clientId",
        scopes: ["cc:clients"],
        bodySchema: Joi.object().keys({
            allowedCallbackUrls: Joi.array(),
            allowedWebOrigins: Joi.array(),
            allowedLogoutUrls: Joi.array(),
            allowedCors: Joi.array()
        }),
        handler: ({ urlParams, principalId, body }) => clientManagementHandler.patchClient({ clientId: urlParams.clientId, userId: principalId, body })
    }];

exports.handler = (event, context, callback) => {
    dispatch({
        event, callback, routes: routes.concat(supportRoutes).concat(developersRoutes), context
    });
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("delete-user", "delete-user", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        deleteUser({ userId: argv.userId, occms: argv.occms });
    })
    .command("delete-associated-occms", "delete-associated-occms", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        deleteOccmClientsCreatedBy({ userId: argv.userId });
    })
    .command("delete-dynamo-entries-for-user", "delete-dynamo-entries-for-user", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        process.env.userInfoTable = "portal-backend-dev-userInfo";
        process.env.userPromoTable = "portal-backend-dev-userPromo";
        deleteDynamoEntriesForUser({ userId: argv.userId }).catch(err => console.log(err));
    }).argv;
