const _ = require("lodash");
const AWS = require('aws-sdk');

const { authAxios } = require("./auth-utils");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const formatName = name => {
    if (!name) { return ""; }
    if (name.length < 20) { return `<b>${name}</b>`; }
    return `<b>${name.substring(0, 20)}...</b>`;
};
// ----------- Storing Notifications -----------

const storeNotification = (userId, visibility, severity, service, message, createDate = new Date().getTime()) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    return dynamoDb.put({
        TableName: process.env.notificationsTable,
        Item: {
            userId,
            createDate,
            visibility,
            severity,
            service,
            message
        }
    }).promise(); // todo try to put in again if same key error
};


const insertAssociateUserNotification = exports.insertAssociateUserNotification = (userId, occmClientId, associatorId, date) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const clientInfoParams = {
        TableName: process.env.occmClientInfoTable,
        Key: { clientId: `${occmClientId}` }
    };
    return authAxios().then(client => Promise.all([client.get(`users/${associatorId}`), dynamoDb.get(clientInfoParams).promise()])).then(arr => {
        const associatorName = _.get(arr[0], "data.user_metadata.name");
        const occmName = _.get(arr[1], "Item.occmName");
        return storeNotification(userId, "new", "info", "ontap-cloud", `You have been added to Cloud Manager ${formatName(occmName)} by ${formatName(associatorName)}`, date)
            .then(() => { console.log("Associate User Notification Stored Successfully"); });
    }).catch(err => {
        console.log("Error. Failed to store associate occm user notification in DB", err);
    });
};


exports.insertDeassociateUserNotification = (userId, occmClientId, deassociatorId, date) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const clientInfoParams = {
        TableName: process.env.occmClientInfoTable,
        Key: { clientId: `${occmClientId}` }
    };
    return authAxios().then(client => Promise.all([client.get(`users/${deassociatorId}`), dynamoDb.get(clientInfoParams).promise()])).then(arr => {
        const deassociatorName = _.get(arr[0], "data.user_metadata.name");
        const occmName = _.get(arr[1], "Item.occmName");
        return storeNotification(userId, "new", "info", "ontap-cloud", `Your user account was removed from ${formatName(occmName)} by ${formatName(deassociatorName)}`, date)
            .then(() => { console.log("Deassociate User Notification Stored Successfully"); });
    }).catch(err => {
        console.log("Error. Failed to store deassociate occm user notification in DB", err);
    });
};


exports.insertRemoveFromListNotification = (userId, occmClientId, date) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const clientInfoParams = {
        TableName: process.env.occmClientInfoTable,
        Key: { clientId: `${occmClientId}` }
    };
    return dynamoDb.get(clientInfoParams).promise().then(clientInfoResponse => {
        const occmName = _.get(clientInfoResponse, "Item.occmName");
        return storeNotification(userId, "new", "info", "ontap-cloud", `You have removed Cloud Manager ${formatName(occmName)} from your Cloud Managers list`, date)
            .then(() => { console.log("Remove from List Notification Stored Successfully"); });
    }).catch(err => {
        console.log("Error. Failed to store remove occm from list notification in DB", err);
    });
};


const insertOccmSetupCompleteNotification = exports.insertOccmSetupCompleteNotification = (userId, occmName) =>
    storeNotification(userId, "new", "success", "ontap-cloud", `Cloud Manager ${formatName(occmName)} has been set up successfully`)
        .then(() => { console.log("OCCM Setup Complete Notification Stored Successfully"); })
        .catch(err => { console.log("Error. Failed to store occm setup complete notification in DB", err); });

const insertAgentSetupCompleteNotification = exports.insertAgentSetupCompleteNotification = (userId, agentName) =>
    storeNotification(userId, "new", "success", "ontap-cloud", `Agent ${formatName(agentName)} has been set up successfully`)
        .then(() => { console.log("Agent Setup Complete Notification Stored Successfully"); })
        .catch(err => { console.log("Error. Failed to store agent setup complete notification in DB", err); });


exports.insertOccmSetupFailedNotification = (userId, occmName) =>
    storeNotification(userId, "new", "error", "ontap-cloud", `Cloud Manager ${formatName(occmName)} setup failed`)
        .then(() => { console.log("Setup Failed Notification Stored Successfully"); })
        .catch(err => { console.log("Error. Failed to store occm setup failed notification in DB", err); });

exports.insertAgentSetupFailedNotification = (userId, agentName) =>
    storeNotification(userId, "new", "error", "ontap-cloud", `Agent ${formatName(agentName)} setup failed`)
        .then(() => { console.log("Setup Failed Notification Stored Successfully"); })
        .catch(err => { console.log("Error. Failed to store agent setup failed notification in DB", err); });


// ----------- Updating Notifications -----------
const setNotificationVisibility = exports.setNotificationVisibility = (userId, createDate, visibility) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const params = {
        TableName: process.env.notificationsTable,
        Key: {
            userId,
            createDate
        },
        UpdateExpression: "SET visibility=:v",
        ExpressionAttributeValues: {
            ":v": visibility
        }
    };
    return dynamoDb.update(params).promise()
        .then(() => {
            console.log("Notification visibility update succeeded");
            return "Notification visibility update succeeded";
        }, err => {
            console.error(`Error updating notification for user ${userId} created at ${createDate} to ${visibility}`, JSON.stringify(err, null, 2));
            throw err;
        });
};


// ----------- Retrieving Notifications -----------

const getUserNotifications = exports.getUserNotifications = userId => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    console.log(`Querying for notifications for user ${userId}`);

    const params = {
        TableName: process.env.notificationsTable,
        KeyConditionExpression: "userId = :i",
        FilterExpression: "visibility =:n or visibility=:o or visibility=:v",
        ExpressionAttributeValues: {
            ":i": userId,
            ":n": "new",
            ":o": "old",
            ":v": "viewed"
        }
    };

    return dynamoDb.query(params).promise()
        .then(data => {
            console.log(`Notifications query succeeded for user ${userId}`);
            return data;
        }, err => {
            console.error("Notifications query failed", JSON.stringify(err, null, 2));
            throw err;
        });
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("insertAssociateUserNotification", "insertAssociateUserNotification", yargs => {
        yargs.demandOption(['userId', 'occmClientId', 'associatorId']);
    }, argv => {
        insertAssociateUserNotification({ userId: argv.userId, occmClientId: argv.occmClientId, associatorId: argv.associatorId });
    })
    .command("getUserNotifications", "getUserNotifications", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        getUserNotifications({ userId: argv.userId });
    })
    .command("setNotificationVisibility", "setNotificationVisibility", yargs => {
        yargs.demandOption(['userId', 'createDate', 'visibility']);
    }, argv => {
        setNotificationVisibility({ userId: argv.userId, createDate: argv.createDate, visibility: argv.visibility });
    })
    .command("insertOccmSetupCompleteNotification", "insertOccmSetupCompleteNotification", yargs => {
        yargs.demandOption(['userId', 'occmName']);
    }, argv => {
        insertOccmSetupCompleteNotification({ userId: argv.userId, occmName: argv.occmName });
    })
    .command("insertAgentSetupCompleteNotification", "insertAgentSetupCompleteNotification", yargs => {
        yargs.demandOption(['userId', 'agentName']);
    }, argv => {
        insertAgentSetupCompleteNotification({ userId: argv.userId, occmName: argv.agentName });
    }).argv;
