const mockAWS = require('aws-sdk-mock');
const nock = require('nock');
const sinon = require('sinon');
const assert = require('assert');

const entry = require("./notifications");
const { authAccount } = require("./auth-utils");

const nockBase = nock(`https://${authAccount.domain}`);

beforeEach(() => {
    nockBase
        .post('/oauth/token')
        .reply(200, { access_token: "mock-access-token" });
});

afterEach(() => {
    mockAWS.restore();
});

// process.env.userInfoTable = "portal-backend-dev-userInfo";
// process.env.occmClientInfoTable = "portal-backend-dev-occmClientInfo";


describe("insertAssociateUserNotification", () => {
    it("should store the notification in dynamo", () => {
        // ---Get User name for notification
        nockBase
            .get("/api/v2/users/associatorId")
            .reply(200, { user_id: "associatorId", user_metadata: { name: "associator name" } });

        // ---Get Client info
        const expectedGetClientParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occmClientId" }
        };
        const getClientInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { clientId: "occmClientId", setupUserId: "setup-user", occmName: "occm name" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientInfoSpy);

        // ---Create entry in notifications Table
        const expectedClientDynamoParams = {
            TableName: process.env.notificationsTable,
            Item: {
                userId: "userId",
                createDate: 111,
                visibility: "new",
                severity: "info",
                service: "ontap-cloud",
                message: "You have been added to Cloud Manager <b>occm name</b> by <b>associator name</b>"
            }
        };
        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);


        return entry.insertAssociateUserNotification("userId", "occmClientId", "associatorId", 111)
            .then(val => {
                assert(getClientInfoSpy.calledWith(expectedGetClientParams), 'should pass correct parameters for get client');
                assert(createClientSpy.calledWith(expectedClientDynamoParams), 'should pass correct parameters in notification creation');
                return expect(val).toBe(undefined);
            });
    });
});


describe("insertDeassociateUserNotification", () => {
    it("should store the notification in dynamo", () => {
        // ---Get User name for notification
        nockBase
            .get("/api/v2/users/dassociatorId")
            .reply(200, { user_id: "dassociatorId", user_metadata: { name: "dassociator name" } });

        // ---Get Client info
        const expectedGetClientParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occmClientId" }
        };
        const getClientInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { clientId: "occmClientId", setupUserId: "setup-user", occmName: "occm name" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientInfoSpy);

        // ---Create entry in notifications Table
        const expectedClientDynamoParams = {
            TableName: process.env.notificationsTable,
            Item: {
                userId: "userId",
                createDate: 111,
                visibility: "new",
                severity: "info",
                service: "ontap-cloud",
                message: "Your user account was removed from <b>occm name</b> by <b>dassociator name</b>"
            }
        };
        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);


        return entry.insertDeassociateUserNotification("userId", "occmClientId", "dassociatorId", 111)
            .then(val => {
                assert(getClientInfoSpy.calledWith(expectedGetClientParams), 'should pass correct parameters for get client');
                assert(createClientSpy.calledWith(expectedClientDynamoParams), 'should pass correct parameters in notification creation');
                return expect(val).toBe(undefined);
            });
    });
});

describe("insertRemoveFromListNotification", () => {
    it("should store the notification in dynamo", () => {
        // ---Get Client info
        const getClientInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { clientId: "occmClientId", setupUserId: "setup-user", occmName: "occm name" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientInfoSpy);

        // ---Create entry in notifications Table
        const expectedClientDynamoParams = {
            TableName: process.env.notificationsTable,
            Item: {
                userId: "userId",
                createDate: 111,
                visibility: "new",
                severity: "info",
                service: "ontap-cloud",
                message: "You have removed Cloud Manager <b>occm name</b> from your Cloud Managers list"
            }
        };
        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);

        return entry.insertRemoveFromListNotification("userId", "occmClientId", 111)
            .then(val => {
                assert(createClientSpy.calledWith(expectedClientDynamoParams), 'should pass correct parameters in notification creation');
                return expect(val).toBe(undefined);
            });
    });
});

describe("setNotificationVisibility", () => {
    it("should update the notification in dynamo", () => {
        // ---Create entry in notifications Table
        const expectedDynamoParams = {
            TableName: process.env.notificationsTable,
            Key: {
                userId: "userId",
                createDate: 111
            },
            UpdateExpression: "SET visibility=:v",
            ExpressionAttributeValues: {
                ":v": "viewed"
            }
        };

        const updateNotificationSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateNotificationSpy);

        return entry.setNotificationVisibility("userId", 111, "viewed")
            .then(val => {
                assert(updateNotificationSpy.calledWith(expectedDynamoParams), 'should pass correct parameters in notification creation');
                return expect(val).toBe("Notification visibility update succeeded");
            });
    });
});
