const mockAWS = require('aws-sdk-mock');
const nock = require('nock');
const sinon = require('sinon');
const assert = require('assert');

const entry = require("./occm-apis");
const { authAccount } = require("./auth-utils");
const { ForbiddenError } = require("./errors");

const nockBase = nock(`https://${authAccount.domain}`);

beforeEach(() => {
    nockBase
        .post('/oauth/token')
        .reply(200, { access_token: "mock-access-token" });
});

afterEach(() => {
    mockAWS.restore();
});

process.env.userInfoTable = "portal-backend-dev-userInfo";
process.env.occmClientInfoTable = "portal-backend-dev-occmClientInfo";

describe("associateUser", () => {
    it("should throw a reasonable error when the email address given doesn't exist", () => {
        nockBase
            .get("/api/v2/users")
            .query({ q: 'email:"non-existent@email"', search_engine: 'v3' })
            .reply(200, []);


        const promise = entry.associateUser({ occmClientId: "occm-client-1", associateeEmail: "non-existent@email", associaterId: "associater-id" });
        return expect(promise).rejects.toHaveProperty("message", 'No user has the given email address: non-existent@email, the user needs to first sign up to Cloud Central (cloud.netapp.com).');
    });

    it("should throw a reasonable error if the email address given is not unique", () => {
        nockBase
            .get("/api/v2/users")
            .query({ q: 'email:"duplicate@email"', search_engine: 'v3' })
            .reply(200, [{}, {}]);

        const promise = entry.associateUser({ occmClientId: "occm-client-1", associateeEmail: "duplicate@email", associaterId: "associater-id" });
        return expect(promise).rejects.toHaveProperty("message", 'Multiple users have the same email address: duplicate@email');
    });

    it("should do nothing if the user is already associated with the occm", () => {
        // ---query Auth0 for user id---
        nockBase
            .get("/api/v2/users")
            .query({ q: 'email:"already-associated@email"', search_engine: 'v3' })
            .reply(200, [{ user_id: "1234" }]);

        // ---update user info---
        const updateUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Attributes: { associatedOccms: { SS: ["occm-client-1"] } } });
        });
        mockAWS.mock('DynamoDB', 'updateItem', updateUserInfoSpy);


        return entry.associateUser({ occmClientId: "occm-client-1", associateeEmail: "already-associated@email", associaterId: "associater-id" })
            .then(val => expect(val).toEqual({ userId: '1234' }));
    });

    it("should modify the user if the user is not already associated with the occm (mergining the array if it already has another element)", () => {
        // ---query Auth0 for user id---
        nockBase
            .get("/api/v2/users")
            .query({ q: 'email:"new@email"', search_engine: 'v3' })
            .reply(200, [{ user_id: "1234" }]);

        // ---update user info---

        const updateUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB', 'updateItem', updateUserInfoSpy);

        // ---Notifications---
        const notificationStub = sinon.stub(entry.Notifications, "insertAssociateUserNotification");
        notificationStub.resolves(null);

        // ---Run Test---
        return entry.associateUser({ occmClientId: "occm-client-12", associateeEmail: "new@email", associaterId: "associater-id" })
            .then(val => {
                // assert(updateUserInfoSpy.calledWith(params), 'should pass correct parameters');
                notificationStub.restore();
                return expect(val).toEqual({ userId: '1234' });
            });
    });
});

describe("deassociateUser", () => {
    it("should remove the occm from the list of associated occms for the user", () => {
        // ---update user info---
        const updateUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Attributes: { associatedOccms: { SS: ["occm-client-1", 'default'] } } });
        });
        mockAWS.mock('DynamoDB', 'updateItem', updateUserInfoSpy);

        // ---Run Test---
        return entry.deassociateUser({
            occmClientId: "occm-client-1",
            deassociaterId: "deassociater-id",
            deassociateeId: "1234"
        })
            .then(val => expect(val).toBeNull());
    });
});

describe("listOccmsOfUser", () => {
    it("should return an empty list if the user has no occms", () => {
        // ---get user info---
        const getUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { userId: "1234" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getUserInfoSpy);

        const promise = entry.listOccmsOfUser({ userId: "1234" });
        return expect(promise).resolves.toEqual({ occms: [], hadPreviousOccm: false });
    });

    it("should return a list of occms if the user has occms", () => {
        // ---get user info---
        const expectedGetUserParams = {
            TableName: process.env.userInfoTable,
            Key: { userId: "1234" }
        };
        const getUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { userId: "1234", associatedOccms: { values: ["occm-client-1", "occm-client-2", "occm-client-3", "occm-client-4", "occm-client-5", "occm-client-6"] }, hadPreviousOccm: true } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getUserInfoSpy);

        // ---batch get all clients---
        const expectedBatchGetParams = {
            RequestItems: {
                [process.env.occmClientInfoTable]: {
                    Keys: [{ clientId: "occm-client-1" }, { clientId: "occm-client-2" }, { clientId: "occm-client-3" }, { clientId: "occm-client-4" }, { clientId: "occm-client-5" }, { clientId: "occm-client-6" }]
                }
            }
        };
        const batchGetSpy = sinon.spy((params, callback) => {
            callback(null, {
                [`Responses.${process.env.occmClientInfoTable}`]: [
                    {
                        clientId: "occm-client-1", automaticCallbackUris: ["http://localhost"], primaryCallbackUri: "http://localhost", occmName: "OCCM Client 1", createDate: 111
                    },
                    { clientId: "occm-client-2", occmName: "OCCM Client 2" },
                    // {clientId: "occm-client-3", primaryCallbackUri: "http://localhost"}, //simulates client that doesnt exist in dynamo
                    {
                        clientId: "occm-client-4", saas: true, createDate: 111, occmName: "Occm Client 4"
                    },
                    {
                        clientId: "occm-client-5", saas: true, setupFailed: true, createDate: 111, occmName: "Occm Client 5"
                    },
                    {
                        clientId: "occm-client-6", automaticCallbackUris: ["http://localhost"], manualOverrideUris: [{ value: "http://localhost2", isDefault: true }], primaryCallbackUri: "http://localhost2", saas: "true", occmName: "OCCM Client 6", setupComplete: "true", createDate: 111
                    }
                ]
            });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'batchGet', batchGetSpy);

        const expected = {
            occms: [
                {
                    occm: "occm-client-1", status: "ready", occmName: "OCCM Client 1", automaticCallbackUris: ["http://localhost"], manualOverrideUris: [], primaryCallbackUri: "http://localhost", createDate: 111
                },
                { occm: "occm-client-2", failed: true, message: "Client does not have a primaryCallbackUri or metadata with an occmName" },
                {
                    occm: "occm-client-4", status: "initializing", createDate: 111, occmName: "Occm Client 4"
                }, // todo added occm name (used to return undefined)
                {
                    occm: "occm-client-5", status: "failed", createDate: 111, occmName: "Occm Client 5"
                }, // todo added occm name (used to return undefined)
                {
                    occm: "occm-client-6", occmName: "OCCM Client 6", automaticCallbackUris: ["http://localhost"], manualOverrideUris: [{ value: "http://localhost2", isDefault: true }], primaryCallbackUri: "http://localhost2", status: "ready", createDate: 111
                }
            ],
            hadPreviousOccm: true
        };
        return entry.listOccmsOfUser({ userId: "1234" })
            .then(val => {
                assert(getUserInfoSpy.calledWith(expectedGetUserParams), 'should pass correct parameters');
                assert(batchGetSpy.calledWith(expectedBatchGetParams), 'should pass correct parameters');
                return expect(val).toEqual(expected);
            });
    });
});

describe("registerOccm", () => {
    it("should return the clientId and secretKey if the occm already exists", () => {
        nockBase.get("/api/v2/clients/occm-client-1")
            .reply(200, { client_secret: "this-is-a-secret" });

        const expectedQueryParams = {
            TableName: process.env.occmClientInfoTable,
            IndexName: 'clientName-Index',
            KeyConditionExpression: 'clientName = :hkey',
            ExpressionAttributeValues: { ':hkey': "Occm-Client-System1" }
        };
        const querySpy = sinon.spy((params, callback) => {
            callback(null, { Items: [{ clientId: "occm-client-1", setupComplete: true }], Count: 1, ScannedCount: 1 });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'query', querySpy);

        return entry.registerOccm({ redirectUris: [], systemId: "System1" })
            .then(val => {
                assert(querySpy.calledWith(expectedQueryParams), 'should pass correct parameters');
                return expect(val).toEqual({
                    clientId: "occm-client-1",
                    clientSecret: "this-is-a-secret",
                    alreadyExists: true,
                    setupComplete: true
                });
            });
    });

    it("should create a new client if the occm does not exist", () => {
        const expectedQueryParams = {
            TableName: process.env.occmClientInfoTable,
            IndexName: 'clientName-Index',
            KeyConditionExpression: 'clientName = :hkey',
            ExpressionAttributeValues: { ':hkey': "Occm-Client-System2" }
        };
        const querySpy = sinon.spy((params, callback) => {
            callback(null, { Items: [], Count: 0, ScannedCount: 0 });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'query', querySpy);


        const newClient = { client_id: "new-client", client_secret: "new-client-secret" };
        nockBase.post("/api/v2/clients", {
            name: "Occm-Client-System2",
            description: "OCCM Client",
            callbacks:
                ["https://localhost",
                    "https://localhost/silent-callback.html"],
            allowed_logout_urls: ["https://localhost"],
            grant_types: ["client_credentials", "implicit", "password"],
            jwt_configuration: { alg: "RS256" },
            app_type: "spa",
            client_metadata: {
                displayName: "Cloud Volumes Ontap"
            }
        })
            .reply(200, newClient);

        nockBase.post("/api/v2/client-grants", {
            client_id: "new-client",
            audience: "https://api.cloud.netapp.com",
            scope: ["occm-access"]
        }).reply(204);

        nockBase.patch("/api/v2/clients/new-client", { token_endpoint_auth_method: 'none' })
            .reply(200, newClient);

        // ---Create entry in occm clients Table

        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);

        return entry.registerOccm({ redirectUris: ["https://localhost", 'http://'], systemId: "System2" })
            .then(val => {
                assert(querySpy.calledWith(expectedQueryParams), 'should pass correct parameters');

                return expect(val).toEqual({
                    clientId: "new-client",
                    clientSecret: "new-client-secret",
                    alreadyExists: false,
                    setupComplete: false
                });
            });
    });
});

describe("occmSetupComplete", () => {
    it("should update the occm client and then associate the user", () => {
        // ---Update client in dynamo
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occm-client-1" },
            AttributeUpdates: {
                setupComplete: { Action: "PUT", Value: true },
                occmName: { Action: "PUT", Value: "Occm Client 1" },
                setupUserId: { Action: "PUT", Value: "user-1" }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);

        // ---Stub all other functions
        const notificationStub = sinon.stub(entry.Notifications, "insertOccmSetupCompleteNotification");
        notificationStub.resolves(null);

        const associateUserInternalStub = sinon.stub(entry, "associateUserInternal");
        associateUserInternalStub.resolves({ userId: "user-1" });

        return entry.occmSetupComplete({
            setupUserId: "user-1",
            occmClientId: "occm-client-1",
            occmName: "Occm Client 1"
        })
            .then(val => {
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                notificationStub.restore();
                associateUserInternalStub.restore();
                return expect(val).toBe(null);
            });
    });
});

describe("manualUpdateOccmUris", () => {
    it("should update the occm client and the database", () => {
        const dynamoGetSpy = sinon.spy((params, callback) => {
            if (params.TableName === 'portal-backend-dev-userInfo') {
                callback(null, { Item: { userId: "user-1", associatedOccms: { values: ["occm-client-1"] } } });
            }
            if (params.TableName === 'portal-backend-dev-occmClientInfo') {
                callback(null, { Item: { automaticCallbackUris: ["http://automatic"] } });
            }
        });

        mockAWS.mock('DynamoDB.DocumentClient', 'get', dynamoGetSpy);

        nockBase
            .patch("/api/v2/clients/occm-client-1", {
                callbacks: [
                    "http://manual1",
                    "http://manual2",
                    "http://automatic",
                    "http://manual1/occmui/",
                    "http://manual2/occmui/",
                    "http://manual1/silent-callback.html",
                    "http://manual2/silent-callback.html",
                    "http://automatic/silent-callback.html",
                    "http://manual1/occmui/silent-callback.html",
                    "http://manual2/occmui/silent-callback.html"],
                allowed_logout_urls: [
                    "http://manual1",
                    "http://manual2",
                    "http://automatic"]
            })
            .reply(204);

        // ---Update client in dynamo
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occm-client-1" },
            AttributeUpdates: {
                automaticCallbackUris: { Action: "PUT", Value: ["http://automatic"] },
                manualOverrideUris: { Action: "PUT", Value: [{ value: "http://manual1", isDefault: true }, { value: "http://manual2", isDefault: false }] }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);


        return entry.manualUpdateOccmUris({
            userId: 'user-1',
            manualOverrideUris: [{ value: "http://manual1", isDefault: true }, {
                value: "http://manual2",
                isDefault: false
            }],
            occmId: "occm-client-1"
        })
            .then(val => {
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                return expect(val).toBe(null);
            });
    });

    it("should not allow to update uri fro occm not associated to the user", () => {
        const dynamoGetSpy = sinon.spy((params, callback) => {
            if (params.TableName === 'portal-backend-dev-userInfo') {
                callback(null, { Item: { userId: "user-1", associatedOccms: { values: ["occm-client-1"] } } });
            }
        });

        mockAWS.mock('DynamoDB.DocumentClient', 'get', dynamoGetSpy);

        return entry.manualUpdateOccmUris({ userId: 'user-id-1', manualOverrideUris: [], occmId: "occm-client-2" })
            .then(null,
                error => expect(error).toEqual(new ForbiddenError('User is not allowed to change OCCM')));
    });
});

describe("update-occm-uris", () => {
    it("should update the occm client and then associate the user", () => {
        nockBase
            .patch("/api/v2/clients/occm-client-1", {
                callbacks: [
                    "http://new1",
                    "http://new2",
                    "http://new1/silent-callback.html",
                    "http://new2/silent-callback.html"],
                allowed_logout_urls: [
                    "http://new1",
                    "http://new2"]
            })
            .reply(204);

        // ---Update client in dynamo
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occm-client-1" },
            AttributeUpdates: {
                automaticCallbackUris: { Action: "PUT", Value: ["http://new1", "http://new2"] },
                manualOverrideUris: { Action: "PUT", Value: [] },
                setupComplete: { Action: "PUT", Value: true }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);

        const getClientSpy = sinon.spy((params, callback) => {
            callback(null, { Item: {} });
        });

        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientSpy);

        return entry.updateOccmUris(({ redirectUris: ["http://new1", "http://new2"], occmClientId: "occm-client-1" }))
            .then(val => {
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                return expect(val).toBe(null);
            });
    });
});

describe("serviceRegisterOccm", () => {
    it("should create a new client", () => {
        const newClient = { client_id: "new-client", client_secret: "new-client-secret" };
        nockBase.post("/api/v2/clients", {
            name: "Occm-Client-System1",
            description: "OCCM Client",
            grant_types: ["client_credentials", "implicit", "password"],
            jwt_configuration: { alg: "RS256" },
            app_type: "spa",
            client_metadata: { displayName: "Cloud Volumes Ontap" }
        })
            .reply(200, newClient);

        nockBase.post("/api/v2/client-grants", {
            client_id: "new-client",
            audience: "https://api.cloud.netapp.com",
            scope: ["occm-access"]
        }).reply(204);

        nockBase.patch("/api/v2/clients/new-client", { token_endpoint_auth_method: 'none' })
            .reply(200, newClient);

        // ---Create entry in occm clients Table
        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);

        // ---update user info (associate user internal)---
        const updateUserInfoSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB', 'updateItem', updateUserInfoSpy);


        // ---Notifications---
        const notificationStub = sinon.stub(entry.Notifications, "insertAssociateUserNotification");
        notificationStub.resolves(null);

        // ---Run Test---
        return entry.serviceRegisterOccm({ occmName: "Johnny's Occm", systemId: "System1", setupUserId: "setup-user" })
            .then(val => {
                notificationStub.restore();
                return expect(val).toEqual({
                    clientId: "new-client",
                    clientSecret: "new-client-secret"
                });
            });
    });
});

describe("serviceSetupFailed", () => {
    it("should update the occm client with the failure flag", () => {
        // ---Get Client info
        const expectedGetClientParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occm-client-1" }
        };
        const getClientInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { clientId: "occm-client-1", setupUserId: "setup-user", occmName: "occm name" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientInfoSpy);

        // ---Update client in dynamo---
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "occm-client-1" },
            AttributeUpdates: {
                setupFailed: { Action: "PUT", Value: true }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);

        // ---Notifications---
        const insertOccmSetupFailedNotificationStub = sinon.stub(entry.Notifications, "insertOccmSetupFailedNotification");
        insertOccmSetupFailedNotificationStub.resolves(null);

        return entry.serviceSetupFailed({ occmClientId: "occm-client-1", setupUserId: "setup-user" })
            .then(val => {
                assert(getClientInfoSpy.calledWith(expectedGetClientParams), 'should pass correct parameters');
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                insertOccmSetupFailedNotificationStub.restore();
                return expect(val).toBe(null);
            });
    });

    it("should throw an error if the user performing the action isn't the setup user of the occm", () => {
        // ---Get Client info
        const getClientInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { clientId: "occm-client-1", setupUserId: "setup-user", occmName: "occm name" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientInfoSpy);

        const promise = entry.serviceSetupFailed({ occmClientId: "occm-client-1", setupUserId: "not-setup-user" });

        return expect(promise).rejects.toHaveProperty("message", 'User not-setup-user does not have permission to affect occm-client-1');
    });
});

describe("getAssociatedUsersOfOccm", () => {
    it("should return an empty list if no users are associated with the given occm", () => {
        nockBase
            .get("/api/v2/users?per_page=100&page=0&include_totals=true&fields=email,user_id,user_metadata&include_fields=true&q=app_metadata.occms:occm-client-0&search_engine=v3")
            .reply(200, {
                start: 0,
                limit: 100,
                length: 0,
                users: [],
                total: 0
            });

        const promise = entry.getAssociatedUsersOfOccm({ occmClientId: "occm-client-0" });

        return expect(promise).resolves.toEqual([]);
    });

    it("should return a list of associated users for this simple occm client (less than 100 associated users)", () => {
        nockBase
            .get("/api/v2/users?per_page=100&page=0&include_totals=true&fields=email,user_id,user_metadata&include_fields=true&q=app_metadata.occms:occm-client-1&search_engine=v3")
            .reply(200, {
                start: 0,
                limit: 100,
                length: 2,
                users: [
                    {
                        email: "user1@netapp.com",
                        user_id: "user_id_1",
                        user_metadata: {
                            name: "User1",
                            company: "NetApp"
                        }
                    },
                    {
                        email: "user2@netapp.com",
                        user_id: "user_id_2",
                        user_metadata: {
                            name: "User2",
                            company: "NetApp"
                        }
                    }
                ],
                total: 2
            });

        const promise = entry.getAssociatedUsersOfOccm({ occmClientId: "occm-client-1" });

        return expect(promise).resolves.toEqual([{
            userId: "user_id_1", name: "User1", email: "user1@netapp.com", company: "NetApp"
        }, {
            userId: "user_id_2", name: "User2", email: "user2@netapp.com", company: "NetApp"
        }]);
    });

    it("should return a list of associated users for this occm client by making 2 API calls to auth0 - 101 users associated", () => {
        nockBase
            .get("/api/v2/users?per_page=100&page=0&include_totals=true&fields=email,user_id,user_metadata&include_fields=true&q=app_metadata.occms:occm-client-2&search_engine=v3")
            .reply(200, {
                start: 0,
                limit: 100,
                length: 100,
                users: [
                    {
                        email: "user1@netapp.com",
                        user_id: "user_id_1",
                        user_metadata: {
                            name: "User1",
                            company: "NetApp"
                        }
                    }
                ],
                total: 101
            });

        nockBase
            .get("/api/v2/users?per_page=100&page=1&include_totals=true&fields=email,user_id,user_metadata&include_fields=true&q=app_metadata.occms:occm-client-2&search_engine=v3")
            .reply(200, {
                start: 100,
                limit: 100,
                length: 1,
                users: [
                    {
                        email: "user2@netapp.com",
                        user_id: "user_id_2",
                        user_metadata: {
                            name: "User2",
                            company: "NetApp"
                        }
                    }
                ],
                total: 101
            });

        const promise = entry.getAssociatedUsersOfOccm({ occmClientId: "occm-client-2" });

        return expect(promise).resolves.toEqual([{
            userId: "user_id_1", name: "User1", email: "user1@netapp.com", company: "NetApp"
        }, {
            userId: "user_id_2", name: "User2", email: "user2@netapp.com", company: "NetApp"
        }]);
    });
});

describe("getOccmUsers", () => {
    it("should return an empty list if no users are associated with the given occm", () => {
        const promise = entry.getOccmUsers({ occmClientId: "occm-client-0", usersString: "" });
        return expect(promise).resolves.toEqual([]);
    });

    it("should return a list of associated users", () => {
        nockBase
            .get("/api/v2/users")
            .query({
                q: 'user_id: user_id_1 OR user_id: user_id_2 OR user_id: user_id_3',
                fields: 'user_id,user_metadata,email',
                include_totals: true,
                search_engine: 'v3'
            })
            .reply(200, {
                length: 2,
                total: 2,
                users: [
                    {
                        email: "user1@netapp.com",
                        user_id: "user_id_1",
                        user_metadata: {
                            name: "User1",
                            company: "NetApp"
                        }
                    }, {
                        email: "user2@netapp.com",
                        user_id: "user_id_2",
                        user_metadata: {
                            name: "User2",
                            company: "NetApp"
                        }
                    }
                ]
            });

        const promise = entry.getOccmUsers({ occmClientId: "occm-client-1", usersString: "user_id_1,user_id_2,user_id_3" });

        return expect(promise).resolves.toEqual([
            {
                userId: "user_id_1", name: "User1", email: "user1@netapp.com", company: "NetApp"
            }, {
                userId: "user_id_2", name: "User2", email: "user2@netapp.com", company: "NetApp"
            }
        ]);
    });
});

describe("getOccmUris", () => {
    it("should return the list of all allowed callback Uris for this occm (without silent callback)", () => {
        const getOccmInfoSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { automaticCallbackUris: ["http://url1", "http://url2", "http://url3"] } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getOccmInfoSpy);

        const promise = entry.getOccmUris({ occmClientId: "occm-client-1" });

        return expect(promise).resolves.toEqual(["http://url1", "http://url2", "http://url3"]);
    });
});

describe("registerAgent", () => {
    it("should create a new client", () => {
        const newClient = { client_id: "new-client", client_secret: "new-client-secret" };
        nockBase.post("/api/v2/clients", {
            name: "Agent-Client-System1",
            description: "Agent Client",
            grant_types: ["client_credentials", "password"],
            jwt_configuration: { alg: "RS256" },
            app_type: "non_interactive",
            client_metadata: {}
        })
            .reply(200, newClient);

        nockBase.post("/api/v2/client-grants", {
            client_id: "new-client",
            audience: "https://api.cloud.netapp.com",
            scope: ["agents:agent"]
        }).reply(204);

        // ---Create entry in occm clients Table
        const createClientSpy = sinon.spy((params, callback) => {
            callback(null, 'success');
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'put', createClientSpy);

        // ---Notifications---
        const notificationStub = sinon.stub(entry.Notifications, "insertAssociateUserNotification");
        notificationStub.resolves(null);

        // ---Run Test---
        return entry.registerAgent({ agentName: "Johnny's Agent", systemId: "System1", setupUserId: "setup-user" })
            .then(val => {
                notificationStub.restore();
                return expect(val).toEqual({
                    clientId: "new-client",
                    clientSecret: "new-client-secret"
                });
            });
    });
});

describe("agentSetupComplete", () => {
    it("should update the occm client", () => {
        // ---Update client in dynamo
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "Agent-client-1" },
            ReturnValues: "ALL_NEW",
            AttributeUpdates: {
                setupComplete: { Action: "PUT", Value: true }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, { Attributes: { setupUserId: "setup-user", occmName: "Johnny's Agent" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);

        // ---Stub all other functions
        const notificationStub = sinon.stub(entry.Notifications, "insertAgentSetupCompleteNotification");
        notificationStub.resolves(null);

        return entry.agentSetupComplete({
            agentClientId: "Agent-client-1"
        })
            .then(val => {
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                notificationStub.restore();
                return expect(val).toBe(null);
            });
    });
});

describe("agentSetupFailed", () => {
    it("should update the occm client", () => {
        // ---get client from dynamo
        const expectedGetParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "Agent-client-1" }
        };
        const getClientSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { occmName: "Johnny's Agent", setupUserId: "setup-user", agent: true } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientSpy);

        // ---Update client in dynamo
        const expectedUpdateParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "Agent-client-1" },
            AttributeUpdates: {
                setupFailed: { Action: "PUT", Value: true }
            }
        };
        const updateClientSpy = sinon.spy((params, callback) => {
            callback(null, "success");
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'update', updateClientSpy);

        // ---Stub all other functions
        const notificationStub = sinon.stub(entry.Notifications, "insertAgentSetupFailedNotification");
        notificationStub.resolves(null);

        return entry.agentSetupFailed({
            agentClientId: "Agent-client-1",
            setupUserId: "setup-user"
        })
            .then(val => {
                assert(getClientSpy.calledWith(expectedGetParams), 'should pass correct parameters');
                assert(updateClientSpy.calledWith(expectedUpdateParams), 'should pass correct parameters');
                notificationStub.restore();
                return expect(val).toBe(undefined);
            });
    });

    it("should not allow update for client which is not agent", () => {
        // ---get client from dynamo
        const expectedGetParams = {
            TableName: process.env.occmClientInfoTable,
            Key: { clientId: "Agent-client-1" }
        };
        const getClientSpy = sinon.spy((params, callback) => {
            callback(null, { Item: { occmName: "Johnny's Agent", setupUserId: "setup-user" } });
        });
        mockAWS.mock('DynamoDB.DocumentClient', 'get', getClientSpy);

        return entry.agentSetupFailed({
            agentClientId: "Agent-client-1",
            setupUserId: "setup-user"
        })
            .then(null, err => {
                assert(getClientSpy.calledWith(expectedGetParams), 'should pass correct parameters');
                return expect(err).toEqual(new ForbiddenError('Client Agent-client-1 is not an agent'));
            });
    });
});
