const _ = require("lodash");
const Joi = require('joi');
const AWS = require('aws-sdk');

const dispatch = require('./dispatch');
const { ResourceNotFoundError } = require("./errors");
const promos = require("./promos/promos");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(null);
const dynamoDb = new AWS.DynamoDB.DocumentClient();

const selectPromo = ({ userId, service, promoId }) => {
    if (promoId) {
        console.log("Specific promoId specified.. ignoring lookup and attempting to retrieve specific promo");
        const promo = promos.find(promo => promo.promoId === promoId); // eslint-disable-line no-shadow
        if (promo) {
            return Promise.resolve(promo);
        }
        console.log(`Specified promoId: ${promoId} doesn not exist - returning random promo`);
        return Promise.resolve(_.sample(promos));
    }

    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: {
            userId: `${userId}`
        }
    };

    const userPromoParams = {
        TableName: process.env.userPromoTable,
        KeyConditionExpression: 'userId = :userId',
        ExpressionAttributeValues: {
            ':userId': userId.toString()
        }
    };

    const startTime = new Date().valueOf();
    return Promise.all([dynamoDb.get(userInfoParams).promise(), dynamoDb.query(userPromoParams).promise()]).then(([userInfoResponse, userPromoResponse]) => {
        const userInfo = userInfoResponse.Item || {};
        const userPromos = userPromoResponse.Items;

        const clickedUserPromos = userPromos.filter(userPromo => userPromo.clicksCount > 0);

        console.log(`promo DB fetched took: ${new Date().valueOf() - startTime}ms,  userInfo: ${JSON.stringify(userInfo)} userPromosCount: ${userPromos.length}. Activating AI.`);

        const notClickedPromos = _.differenceBy(promos.filter(promo => !!promo.enabled), clickedUserPromos, "promoId");

        const filteredPromos = notClickedPromos.filter(promo => {
            if (promo.serviceKey === service) { return false; }

            if (promo.notApply && promo.notApply.length > 0) {
                const userNotApply = promo.notApply.find(el => userInfo[el]);

                if (userNotApply) { return false; }
            }

            return true;
        });


        const selectedPromo = _.sample(filteredPromos);

        if (selectedPromo) {
            const selectedUserPromo = userPromos.find(el => el.promoId === selectedPromo.promoId);

            if (selectedUserPromo) {
                selectedPromo.viewsCount = selectedUserPromo.viewsCount;
            } else if (selectedPromo) {
                selectedPromo.viewsCount = 0;
            }

            return selectedPromo;
        }
        return null;
    }, err => {
        console.error("Unable to fetch user - promo - information from db - returning random promo", JSON.stringify(err, null, 2));
        return _.sample(promos);
    });
};

const fillInLink = (promo, userId) => {
    promo.linkUrl = `${process.env.portalBackEndUrl}/analytics/promo/${promo.promoId}?track=${encodeURIComponent(Buffer.from(userId).toString('base64'))}`;
};

const getPromo = ({ userId, service, promoId }) => {
    console.log(`Getting promo for user: ${userId} when using service: ${service}`);

    return selectPromo({ userId, service, promoId }).then(selectedPromo => {
        if (selectedPromo) {
            fillInLink(selectedPromo, userId);
            return selectedPromo;
        }
        return null;
    });
};


const trackView = ({ promoId, userId }) => {
    console.log(`Tracking view for promo ${promoId} to user ${userId}`);
    const params = {
        TableName: process.env.userPromoTable,
        Key: { userId, promoId },
        UpdateExpression: 'ADD viewsCount :val',
        ExpressionAttributeValues: {
            ":val": 1
        }
    };

    return dynamoDb.update(params).promise().then(() => null, err => {
        console.log("Warning: failed to update view count for promo event");
        console.log(err);
        return null;
    });
};

exports.redirectPromoLink = ({ promoId, track }) => {
    const userId = Buffer.from(decodeURIComponent(track), 'base64').toString();
    console.log(`Tracking promo link click for promo: ${promoId} and user: ${userId}`);
    const promo = promos.find(promo => promo.promoId === promoId); // eslint-disable-line no-shadow

    if (promo) {
        const params = {
            TableName: process.env.userPromoTable,
            Key: { userId, promoId: promo.promoId },
            UpdateExpression: 'ADD clicksCount :val',
            ExpressionAttributeValues: {
                ":val": 1
            }
        };

        const response = { statusCode: 307, link: promo.linkUrl };
        return dynamoDb.update(params).promise().then(() => response, err => {
            console.log("Warning: failed to update click count for promo event");
            console.log(err);
            return response;
        });
    }
    return Promise.reject(new ResourceNotFoundError(`The given promo ${promoId} does not exist.`));
};

const routes = [{
    method: "GET",
    pattern: "/promo/feeling-lucky",
    querySchema: Joi.object().keys({
        service: Joi.string()
    }),
    handler: ({ principalId, queryParams }) => getPromo({ userId: principalId, service: queryParams.service, promoId: queryParams.promoId }),
    audit: {
        include: true
    }
}, {
    method: "POST",
    pattern: "/promo/track-view/:promoId",
    handler: ({ principalId, urlParams }) => trackView({ userId: principalId, promoId: urlParams.promoId })
}];

exports.handler = (event, context, callback) => {
    dispatch({
        event, callback, routes, context
    });
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("get-promo", "get-promo", yargs => {
        yargs.demandOption(["userId", "service"]);
    }, argv => {
        getPromo({ userId: argv.userId, service: argv.service, promoId: argv.promoId }).then(promo => console.log(promo));
    }).argv;
