const _ = require("lodash");
const moment = require('moment');
const AWS = require('aws-sdk');

const { authAxios } = require("./../auth-utils");
const { sendInternalEmail } = require('./../utils/internal-email');
const { internalDevTeamMailList } = require("./../consts");
const { getLogsRecursive, storeAuth0Checkpoint, getAuth0Checkpoint } = require('./auth0-logs-fetcher');

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));


const pushLogsToS3 = (logArray, yesterdayString) => {
    const lastEntryId = logArray[logArray.length - 1]._id || logArray[logArray.length - 1].log_id; // eslint-disable-line no-underscore-dangle
    const fileName = `${process.env.stage}/auth0-${process.env.stage}-${yesterdayString}.log`;
    const binaryFile = Buffer.from(JSON.stringify(logArray), 'binary');
    const params = {
        Bucket: 'auth0-logs.portal.netapp.com',
        Key: fileName,
        Body: binaryFile
    };
    const s3 = new AWS.S3();
    return s3.putObject(params).promise().then(res => {
        console.log(`Insertion to S3 completed successfully. Stored ${logArray.length} logs. ETag: ${_.get(res, 'ETag')}`);
        return lastEntryId;
    });
};

const logsToS3 = () => {
    const yesterdayString = moment().subtract(1, 'days').format('YYYY-MM-DD');
    console.log(`Getting logs from Auth0 for the date: ${yesterdayString} in env ${process.env.stage}`);

    return getAuth0Checkpoint("auth0CheckpointForS3").then(({ value }) => authAxios().then(client => getLogsRecursive([], value, 0, client)
        .then(logArray => {
            const filteredArray = logArray.filter(log => log.date.startsWith(yesterdayString));
            if (filteredArray.length === 0) {
                console.log("No new logs from Auth0. Exiting.");
                return null;
            }
            return pushLogsToS3(filteredArray, yesterdayString)
                .then(lastEntryId => storeAuth0Checkpoint(lastEntryId, 'auth0CheckpointForS3'));
        })
        .then(() => "Success"))).catch(err => {
        const message = `Failed to push Auth0 logs in ${process.env.stage} env to S3. See cloud watch logs for more details`;
        return sendInternalEmail("Failed to push Auth0 logs to S3", message, internalDevTeamMailList).then(() => {
            console.log("Error");
            console.log(err);
            throw err;
        }, mailErr => {
            console.log("Error getting logs and sending internal email");
            console.log(mailErr);
            throw mailErr;
        });
    });
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("pushLogs", "pushLogs", _.noop, () => {
        pushLogsToS3();
    })
    .command("logsToS3", "logsToS3", _.noop, () => {
        logsToS3().then(res => console.log(res), () => process.exit(1));
    }).argv;
