const zlib = require('zlib');
const fs = require('fs');
const Promise = require('bluebird');
const axios = require('axios');
const AWS = require('aws-sdk'); // eslint-disable-line import/no-extraneous-dependencies
const { authAxios } = require("./../../auth-utils");

AWS.config.update({ region: 'us-east-1' });
const dynamoDb = new AWS.DynamoDB.DocumentClient();
const s3 = new AWS.S3();

const readPreviousResultsFromDynamo = () => {
    console.log(`getting NetApp Users from misc table`);
    const params = {
        TableName: process.env.miscTable,
        Key: { key: 'auth0NetAppUsers' }
    };
    return dynamoDb.get(params).promise().then(res => (res.Item ? res.Item.value : 0));
};

const getUsersJob = () => {
    console.log(`Executing user export job for connection ${process.env.samlConnectioId}`);
    return authAxios().then(authAxioClient => authAxioClient.post('jobs/users-exports', {
        connection_id: process.env.samlConnectioId,
        format: 'json',
        limit: 100,
        fields: [{ name: 'email' }, { name: 'user_id' }, { name: 'name' }]
    })
        .then(res => {
            const jobId = res.data.id;
            console.log(`Job Id: ${jobId}`);
            // eslint-disable-next-line no-shadow
            const poll = () => authAxios().then(authAxioClient => authAxioClient.get(`jobs/${jobId}`)
                .then(pollRes => {
                    console.log(`Job polling response status: ${pollRes.data.status}`);
                    if (pollRes.data.status === "completed") {
                        return pollRes.data.location;
                    }
                    Promise.delay(700);
                    return poll();
                }));
            return poll();
        })
        .then(url => axios({ method: 'get', url, responseType: 'stream' }))
        .then(res => res.data.pipe(zlib.createGunzip()))
        .then(readStream => {
            const chunks = [];
            return new Promise((resolve, reject) => {
                readStream.on('data', chunk => chunks.push(chunk));
                readStream.on('error', reject);
                readStream.on('end', () => resolve(Buffer.concat(chunks).toString('utf8')));
            });
        })
        .then(s => JSON.parse(`[${s.replace(/\r?\n|\r/g, ',').slice(0, -1)}]`)));
};

const filterNetAppEmails = users => users.filter(user => /@netapp.com$/.test(user.email));

const updateNumberOfUsersInDynamo = numUsers => {
    console.log(`Updating misc table NetApp users info`);
    const params = {
        TableName: process.env.miscTable,
        Item: {
            key: 'auth0NetAppUsers',
            value: numUsers,
            date: new Date().getTime()
        }
    };
    return dynamoDb.put(params).promise()
        .then(() => numUsers);
};

const calculateGrowth = (prev, current) => (prev === current ? 0 : parseFloat((prev / current) * 100).toFixed(2));

const generateHtml = data => new Promise((resolve, reject) => {
    fs.readFile(`${__dirname}/netappUsersReportTemplate.html`, 'utf8', (err, fileText) => {
        if (err) {
            reject(err);
        } else {
            let html = fileText.replace(/ <!-- users -->/g, data.current);
            html = html.replace(/ <!-- growth -->/g, data.growth);
            resolve(html);
        }
    });
});

const writeResultsToBucket = html => {
    console.log(`Creating object in S3 ${process.env.privateBucket}`);
    const bucketParams = {
        Bucket: process.env.privateBucket,
        Body: Buffer.from(html, 'binary'),
        Key: 'reports/netapp-users/netappUsersReport.html'
    };
    return s3.putObject(bucketParams).promise();
};


exports.handler = (event, context, callback) => {
    console.log("Attempting to generate NetApp users report");

    const data = {};

    return readPreviousResultsFromDynamo()
        .then(value => {
            data.prev = value;
            return getUsersJob();
        })
        .then(users => {
            const netappEmailUsers = filterNetAppEmails(users);
            console.log(`Found ${netappEmailUsers.length} netapp users`);
            data.current = netappEmailUsers.length;
            return updateNumberOfUsersInDynamo(netappEmailUsers.length);
        })
        .then(() => {
            data.growth = calculateGrowth(data.prev, data.current);
            console.log(`Growth of ${data.growth}%`);
            return generateHtml(data);
        })
        .then(writeResultsToBucket)
        .then(() => callback(null, {}))
        .catch(err => {
            console.log("Error");
            console.log(err);
            return callback(null, {});
        });
};
