const axios = require('axios');
const util = require('util');
const AWS = require('aws-sdk');

/*
    AWS notification states
        subscribe-success => subscription succeeded
        subscribe-fail => subscription failed
        unsubscribe-pending => unsubscribed, allowed to charge for one more hour
        ubsubscribe-success => unsubscribed, don't charge anymore
        entitlement-updated => only for saas contracts

    SS notification states
        pending => received redirect but not SNS
        subscribed => subscription success from SNS
        failed => subscription failed from SNS
        unsubscribe-pending => received unsubscribe-pending from SNS
        unsubscribed => received unsubscrib-success from SNS


    SubscriptionTable
    subscriptionId (cloudProvider-productCode-customerId), accounts: [accountIds], cloudProvider, productCode, customerId, subscriptionName, subscription Status, serviceName, lastSubscriptionStatus

    ProductsTable
    productCode serviceName sns
 */


const awsStatusMapper = {
    "subscribe-success": "subscribed",
    "subscribe-fail": "failed",
    "unsubscribe-pending": "unsubscribe-pending",
    "unsubscribe-success": "unsubscribed"
};

const handleSubscriptionConfirmation = async record => {
    console.log("attempting to call subscribe url");
    try {
        await axios.get(record.SubscribeURL);
        console.log("Successfully confirmed subscription request");
    } catch (e) {
        console.log("Failed to subscribe subscription request");
        console.log(e);
    }
    return null;
};

const handleNotification = async record => {
    console.log("handling notification");
    try {
        const message = JSON.parse(record.Message);
        console.log("message contents:");
        console.log(message);

        if (!message['product-code'] || !message['customer-identifier'] || !message.action) {
            throw new Error('Malformed message - unable to handle notification');
        }

        const documentClient = new AWS.DynamoDB.DocumentClient();
        const subscriptionId = `aws-${message['product-code']}-${message['customer-identifier']}`;

        console.log("validating product code");
        const productParams = {
            TableName: process.env.productsTable,
            Key: { productCode: message['product-code'] }
        };

        const status = awsStatusMapper[message.action];
        if (!status) {
            throw new Error(`Invalid action - ${message.action}`);
        }

        const productResponse = await documentClient.get(productParams).promise();
        if (!productResponse.Item) {
            throw new Error(`Invalid product code - ${message['product-code']}`);
        }

        console.log("updating database entry");
        const updateParams = {
            TableName: process.env.subscriptionsTable,
            Key: { subscriptionId },
            ReturnValues: "ALL_NEW",
            UpdateExpression: 'SET cloudProvider = :cp, subscriptionStatus = :s, serviceName = :sn, customerIdentifier = :ci, productCode = :pc, createdAt = if_not_exists(createdAt, :ca)',
            ExpressionAttributeValues: {
                ':cp': "aws",
                ':s': status,
                ':sn': productResponse.Item.serviceName,
                ':ci': message['customer-identifier'],
                ':pc': message['product-code'],
                ':ca': new Date().valueOf()
            }
        };

        const updateResponse = await documentClient.update(updateParams).promise();
        console.log("Successfully updated database entry - new entry:");
        console.log(updateResponse);
        return null;
    } catch (e) {
        console.log("!!!CRITICAL!!! Failed to handle notification");
        console.log(e);
    }
    return null;
};

const handleRecordRaw = recordRaw => {
    try {
        console.log("Parsing record");
        const record = JSON.parse(recordRaw.body);
        console.log("record contents:");
        console.log(util.inspect(record, { showHidden: false, depth: null }));
        if (record.Type === "SubscriptionConfirmation") {
            return handleSubscriptionConfirmation(record);
        } if (record.Type === "Notification") {
            return handleNotification(record);
        }
        console.log("Unhandled subscription type");
    } catch (e) {
        console.log(`Unable to handle record`);
        console.log(recordRaw);
    }
    return Promise.resolve(null);
};

exports.handler = (event, context, callback) => {
    console.log('Received queue chunk');
    console.log(event);
    const { Records } = event;

    if (!Records) {
        console.log('No records present.. quitting');
    } else {
        console.log(`Records length ${Records.length}`);
        Records.map(recordRaw => handleRecordRaw(recordRaw));
    }

    callback(null, {});
};
