const { exec } = require('child_process');
const { argv } = require('yargs');
const { removeSync, copySync, writeFileSync } = require('fs-extra');

const REGISTRY_DEVELOPMENT = 'cloudmanagerinfra.azurecr.io';
const REPOSITORY = `${REGISTRY_DEVELOPMENT}/charging`;

const BUILD_DIRECTORY = 'build';
const SRC_DIRECTORY = 'src';

async function execute(command, cwd = undefined) {
    console.log('Executing command:', { command, cwd });
    return new Promise((resolve, reject) => {
        const childProcess = exec(command, { cwd }, (error, stdout, stderr) => {
            if (error) {
                reject(stderr || stdout);
            } else {
                resolve(stdout.trim());
            }
        });

        childProcess.stdout.on('data', data => console.log(data));
        childProcess.stderr.on('data', data => console.error(data));
    });
}

async function pushDocker(tags, registry) {
    await execute(`echo ${argv.password} | docker login -u ${argv.username} ${registry} --password-stdin`);
    await Promise.all(tags.map(tag => execute(`docker push ${tag}`)));
    await execute('docker logout');
}

async function getShortRevision() {
    return execute('git rev-parse --short HEAD');
}

async function main() {
    try {
        console.log('Starting...', argv);
        console.log('#1 - cleaning build directory');
        removeSync(BUILD_DIRECTORY);

        console.log('#2 - copying server assets');
        copySync(`../../package.json`, `${BUILD_DIRECTORY}/package.json`);
        copySync(`../../yarn.lock`, `${BUILD_DIRECTORY}/yarn.lock`);
        copySync(`../../${SRC_DIRECTORY}`, `${BUILD_DIRECTORY}`);

        const shortRevision = await getShortRevision();
        const label = `charging-${shortRevision}-${argv.buildCounter}`;
        // write about file for TC artifact
        writeFileSync(`${BUILD_DIRECTORY}/about.json`, JSON.stringify({ version: label }, null, 4));

        const tags = [`${REPOSITORY}:${label}`, `${REPOSITORY}:latest`];
        console.log('#3 - building images', tags);
        await execute(`docker build --tag ${tags[0]} --tag ${tags[1]} .`);

        console.log('#4 - pushing images');
        await pushDocker(tags, REGISTRY_DEVELOPMENT);
        console.log('#5 - delete previous images');
        // await execute(`docker rmi ${tags[0]}`);

        console.log('Done !!!');
    } catch (err) {
        console.error('Failed to build', err);
        process.exit(-1);
    }
}

main();
