const express = require('express');
const _ = require("lodash");
const { validateRequestBody, addMergedId, validateUniqueness, calculateTotals } = require("../middleware/hourlyCharge");
const { resolveSubscriptions, saveToHourlyTable, reportMetering, updateHourlyTable, updateAccumulatedCharges } = require("../services/hourlyCharge");
const { getSubscription, saveHourlyCharge, updateHourlyCharge, getAccumulatedCharge, updateAccumulatedCharge, saveAccumulatedCharge } = require("../services/database");
const metering = require("../services/metering");
const { getLogger } = require("../logger");

const logger = getLogger();

const router = express.Router();

router.post("/charging", validateRequestBody);
router.post("/charging", addMergedId);
router.post("/charging", validateUniqueness);
router.post("/charging", calculateTotals);

const handleSubscriptionResolutionErrors = resolvedHourlyCharges => {
    const [resolvedWithError, resolvedWithoutError] = _.partition(resolvedHourlyCharges, 'error');
    logger.error(`!!!CRITICAL!!! Encountered error on resolution attempt: ${JSON.stringify(resolvedWithError)}`);
    return resolvedWithoutError;
};

const handleMeteringReportErrors = reportedHourlyCharges => {
    const [reportedWithoutError, reportedWithError] = _.partition(reportedHourlyCharges, charge => (charge.chargingStatus === "success" || charge.chargingStatus === "mock"));
    logger.error(`!!!CRITICAL!!! Encountered error on reporting attempt: ${JSON.stringify(reportedWithError)}`);
    return reportedWithoutError;
};

// eslint-disable-next-line no-unused-vars
router.post('/charging', (req, res, next) => {
    resolveSubscriptions(getSubscription)(req.body)
        .then(handleSubscriptionResolutionErrors)
        .then(saveToHourlyTable(saveHourlyCharge))
        .then(reportMetering(metering.reportMetering))
        .then(updateHourlyTable(updateHourlyCharge))
        .then(handleMeteringReportErrors)
        .then(updateAccumulatedCharges(getAccumulatedCharge, updateAccumulatedCharge, saveAccumulatedCharge));
    res.end("Accepted Charges");
});

// eslint-disable-next-line no-unused-vars
router.get("/health", async (req, res, next) => {
    logger.info("Checking server health");
    getSubscription("fake-subscription-id")
        .then(() => {
            res.status = 203;
            res.send(`Server ready`);
        })
        .catch(err => {
            const error = new Error(`Health check failed with error ${err}`);
            error.httpStatusCode = 503;
            return next(error);
        });
});

module.exports = router;
