const AWS = require('aws-sdk/index');
const { processTimestamp, fromUnixToIsoString, doWithRetry } = require("../utils");
const { getLogger } = require("../logger");

const logger = getLogger();
AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

AWS.config.update({ accessKeyId: process.env.accessKey, secretAccessKey: process.env.secretKey });

const dynamoClient = new AWS.DynamoDB.DocumentClient();

const shouldRetry = ({ code, statusCode }, retries) => (
    code === "ItemCollectionSizeLimitExceededException"
    || code === "LimitExceededException"
    || code === "ProvisionedThroughputExceededException"
    || code === "RequestLimitExceeded"
    || code === "ThrottlingException"
    || code === "UnrecognizedClientException"
    || statusCode == 500 // eslint-disable-line eqeqeq
    || statusCode == 503 // eslint-disable-line eqeqeq
) && (retries < process.env.retries);

const getSubscription = subscriptionId => {
    logger.info(`Attempting to resolve subscription ${subscriptionId}`);
    const params = {
        TableName: process.env.subscriptionsTable,
        Key: { subscriptionId }
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.get(params).promise()
        .then(response => {
            logger.info(`SubscriptionId response: ${JSON.stringify(response.Item || {})}`);
            return (response.Item || {});
        }));
};

const getAccumulatedCharge = ({ mergedId, timestamp }) => {
    logger.info(`Attempting to get accumulated charge ${JSON.stringify({ mergedId, timestamp })}`);
    const timestampByDate = processTimestamp(timestamp, "daily");
    const params = {
        TableName: process.env.accumulatedBillingTable,
        Key: { mergedId, timestamp: timestampByDate }
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.get(params).promise()
        .then(response => {
            logger.info(`Retrieved accumulated charge successfully: ${JSON.stringify(response.Item || {})}`);
            return (response.Item || {});
        }));
};

const saveHourlyCharge = ({ mergedId, subscriptionId, customerIdentifier, dimension, timestamp, accounts, cloudProvider, productCode, serviceName, accumulatedVal, error }) => {
    logger.info(`Attempting to save charge ${JSON.stringify({ mergedId, subscriptionId, customerIdentifier, dimension, timestamp, accounts, cloudProvider, productCode, serviceName, accumulatedVal, error })}`);
    const timestampByDateAndHour = processTimestamp(timestamp, "hourly");
    const params = {
        TableName: process.env.billingTable,
        Item: {
            mergedId,
            subscriptionId,
            timestamp: timestampByDateAndHour,
            isoStringTimestamp: fromUnixToIsoString(timestampByDateAndHour),
            customerIdentifier,
            dimension,
            accounts,
            cloudProvider,
            productCode,
            serviceName,
            accumulatedVal,
            error
        },
        ExpressionAttributeNames: { "#timestamp": "timestamp" },
        ConditionExpression: "attribute_not_exists(mergedId) AND attribute_not_exists(#timestamp)"
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.put(params).promise()
        .then(response => {
            logger.info(`Charge saved successfully`);
            return response;
        }));
};

const saveAccumulatedCharge = ({ mergedId, accountId, subscriptionId, customerIdentifier, dimension, timestamp, cloudProvider, productCode, serviceName, totalVal, resources }) => {
    logger.info(`Attempting to save accumulated charge ${JSON.stringify({ mergedId, accountId, subscriptionId, dimension, timestamp, cloudProvider, productCode, serviceName, totalVal, resources })}`);
    const timestampByDate = processTimestamp(timestamp, "daily");
    const params = {
        TableName: process.env.accumulatedBillingTable,
        Item: {
            mergedId,
            timestamp: timestampByDate,
            isoStringTimestamp: fromUnixToIsoString(timestampByDate),
            subscriptionId,
            customerIdentifier,
            dimension,
            accountId,
            cloudProvider,
            productCode,
            serviceName,
            totalVal,
            resources
        },
        ExpressionAttributeNames: { "#timestamp": "timestamp" },
        ConditionExpression: "attribute_not_exists(mergedId) AND attribute_not_exists(#timestamp)"
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.put(params).promise()
        .then(response => {
            logger.info(`Accumulated charge saved successfully`);
            return response;
        }));
};

const updateHourlyCharge = ({ mergedId, timestamp, chargingStatus }) => {
    logger.info(`Attempting to update hourly charge ${JSON.stringify({ mergedId, timestamp, chargingStatus })}`);
    const timestampByDateAndHour = processTimestamp(timestamp, "hourly");
    const params = {
        TableName: process.env.billingTable,
        Key: { mergedId, timestamp: timestampByDateAndHour },
        UpdateExpression: "set chargingStatus = :status",
        ExpressionAttributeValues: {
            ":status": chargingStatus
        },
        ReturnValues: "ALL_OLD"
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.update(params).promise()
        .then(response => {
            logger.info(`Updated hourly charge successfully: ${JSON.stringify(response.Attributes || {})}`);
            return (response.Attributes || {});
        }));
};

const updateAccumulatedCharge = ({ mergedId, timestamp, resources, totalVal }) => {
    logger.info(`Attempting to update accumulated charge ${JSON.stringify({ mergedId, timestamp, resources, totalVal })}`);
    const timestampByDate = processTimestamp(timestamp, "daily");
    const params = {
        TableName: process.env.accumulatedBillingTable,
        Key: { mergedId, timestamp: timestampByDate },
        UpdateExpression: "set resources = :resources, totalVal = :totalVal",
        ExpressionAttributeValues: {
            ":resources": resources,
            ":totalVal": totalVal
        },
        ReturnValues: "ALL_OLD"
    };
    return doWithRetry(shouldRetry)(0, () => dynamoClient.update(params).promise()
        .then(response => {
            logger.info(`Updated accumulated charge successfully: ${JSON.stringify(response.Attributes || {})}`);
            return (response.Attributes || {});
        }));
};

exports.doWithRetry = doWithRetry;
exports.getSubscription = getSubscription;
exports.saveHourlyCharge = saveHourlyCharge;
exports.saveAccumulatedCharge = saveAccumulatedCharge;
exports.getAccumulatedCharge = getAccumulatedCharge;
exports.updateAccumulatedCharge = updateAccumulatedCharge;
exports.updateHourlyCharge = updateHourlyCharge;
exports.shouldRetry = shouldRetry;
