const _ = require("lodash");
const Promise = require("bluebird");
const { promiseInspect } = require("../utils");
const { getLogger } = require("../logger");

const logger = getLogger();

const totalPerAccount = resources => _.sumBy(resources, 'val');

const totalPerSubscription = accounts => {
    logger.info(`Calculating hourly charge for ${JSON.stringify(accounts)}`);
    return accounts.reduce(({ accumulatedVal, accumulatedAccounts }, account) => {
        const sumPerAccount = totalPerAccount(account.resources);
        return {
            accumulatedVal: accumulatedVal + sumPerAccount,
            accumulatedAccounts: _.concat(accumulatedAccounts, [{ ...account, totalVal: sumPerAccount }])
        };
    }, { accumulatedVal: 0, accumulatedAccounts: [] });
};

const resolveSubscription = get => subscriptionId => promiseInspect(get(subscriptionId));

const saveHourlyCharge = save => charge => promiseInspect(save(charge));

const updateHourlyChargeStatus = update => charge => promiseInspect(update(charge));

const resolveSubscriptions = getSubscriptionFunc => hourlyCharges => {
    logger.info(`RESOLVING SUBSCRIPTIONS`);
    return Promise.map(hourlyCharges, async charge => {
        const enhancedChargeObj = { ...charge };
        const resolution = await promiseInspect(getSubscriptionFunc(enhancedChargeObj.subscriptionId));
        if ((resolution.status === "fulfilled" && _.isEmpty(resolution.value)) || resolution.status === "rejected") {
            logger.error(`!!!CRITICAL!!! Could not resolve subscription id`);
            enhancedChargeObj.error = (resolution.status === "rejected") ? resolution.error : "subscription-not-found";
            return enhancedChargeObj;
        }
        if ((resolution.value.subscriptionStatus !== "subscribed") && (resolution.value.subscriptionStatus !== "unsubscribed-pending")) {
            logger.error(`!!!CRITICAL!!! Subscription status error: ${resolution.value.subscriptionStatus}`);
            enhancedChargeObj.error = `subscription-status error: ${resolution.value.subscriptionStatus}`;
            return enhancedChargeObj;
        }
        return { ...enhancedChargeObj, ..._.pick(resolution.value, ['cloudProvider', 'customerIdentifier', 'productCode', 'serviceName', 'subscriptionStatus']) };
    }, { concurrency: 1 });
};

const saveToHourlyTable = saveInDatabaseFunc => resolvedHourlyCharges => {
    logger.info(`SAVING HOURLY CHARGES`);
    return Promise.map(resolvedHourlyCharges, async charge => {
        const savedChargeObj = { ...charge };
        const saveToHourlyTableRes = await promiseInspect(saveInDatabaseFunc(savedChargeObj));
        if (saveToHourlyTableRes.status === "rejected") {
            logger.warn(`Could not save intent to charge to hourly table`);
            savedChargeObj.error = saveToHourlyTableRes.error;
            return savedChargeObj;
        }
        return savedChargeObj;
    }, { concurrency: 1 });
};

const processMeteringResponse = (enhancedChargeBatch, reportResponse) => {
    logger.info(`Processing response for batch: ${JSON.stringify(enhancedChargeBatch)}`);
    logger.info(`In metering response: ${JSON.stringify(reportResponse)}`);
    return _.map(enhancedChargeBatch, charge => {
        if (reportResponse.status === "rejected") {
            logger.error(`!!!CRITICAL!!! Metering response was not processed for ${charge}`);
            return { ...charge, chargingStatus: "fail" };
        }
        const processedMeteringResponse = _.find(reportResponse.value.Results, {
            UsageRecord: {
                Quantity: charge.accumulatedVal,
                Timestamp: charge.timestamp,
                Dimension: charge.dimension,
                CustomerIdentifier: charge.customerIdentifier
            }
        });
        if (!processedMeteringResponse) {
            logger.error(`!!!CRITICAL!!! Metering response was not processed for ${charge}`);
            return { ...charge, chargingStatus: "fail" };
        }
        return { ...charge, chargingStatus: (processedMeteringResponse.Status === "Success") ? "success" : "fail" };
    });
};

const meteringStubFunc = enhancedHourlyCharges => {
    logger.info("STUB METERING");
    const c = _.map(enhancedHourlyCharges, charge => ({ ...charge, chargingStatus: "mock" }));
    return Promise.resolve(c);
};

const reportMetering = report => enhancedHourlyCharges => {
    logger.info(`REPORTING TO METERING`);
    if (process.env.doCharge !== 'true') {
        return meteringStubFunc(enhancedHourlyCharges);
    }
    const groupedByProduct = _.groupBy(enhancedHourlyCharges, 'productCode');
    logger.info(`Performing metering for: ${JSON.stringify(groupedByProduct)}`);
    return Promise.map(_.keys(groupedByProduct), product => {
        logger.info(`Metering product: ${product}`);
        const enhancedChargeBatches = _.chunk(groupedByProduct[product], 25);
        return Promise.map(enhancedChargeBatches, async enhancedChargeBatch => {
            const usages = _.map(enhancedChargeBatch, enhancedCharge => ({
                CustomerIdentifier: enhancedCharge.customerIdentifier,
                Dimension: enhancedCharge.dimension,
                Quantity: enhancedCharge.accumulatedVal,
                Timestamp: enhancedCharge.timestamp
            }));
            logger.info(`Metering batch: ${JSON.stringify(usages)}`);
            const reportResponse = await promiseInspect(report(product, usages));
            return processMeteringResponse(enhancedChargeBatch, reportResponse);
        }, { concurrency: 1 });
    }, { concurrency: 1 })
        .then(response => _.flatMapDeep(response));
};

const updateHourlyTable = updateDatabaseFunc => reportedHourlyCharges => {
    logger.info(`UPDATING HOURLY CHARGE WITH METERING STATUS`);
    return Promise.map(reportedHourlyCharges, async charge => {
        logger.info(`Updating charge: ${JSON.stringify(charge)}`);
        const updatedChargeObj = { ...charge };
        const saveToHourlyTableRes = await promiseInspect(updateDatabaseFunc(updatedChargeObj));
        if ((saveToHourlyTableRes.status === "fulfilled" && _.isEmpty(saveToHourlyTableRes.value)) || saveToHourlyTableRes.status === "rejected") {
            logger.info(`Critical error!!! Could not update charge's chargingStatus in hourly table`);
            updatedChargeObj.error = (saveToHourlyTableRes.status === "rejected") ? saveToHourlyTableRes.error : "Failed to update accumulated charge - original item not found!";
            return updatedChargeObj;
        }
        return updatedChargeObj;
    }, { concurrency: 1 });
};

const convertToChargesPerAccount = hourlyCharge => _.map(hourlyCharge.accounts, account => ({
    mergedId: `${hourlyCharge.subscriptionId}-${hourlyCharge.dimension}-${account.accountId}`,
    ..._.pick(hourlyCharge, ['customerIdentifier', 'subscriptionId', 'cloudProvider', 'dimension', 'productCode', 'serviceName', 'timestamp']),
    accountId: account.accountId,
    resources: account.resources,
    totalVal: account.totalVal
}));

// TODO: test
const updateAccumulatedCharges = (getChargeFunc, updateDatabaseFunc, saveToDatabaseFunc) => updatedHourlyCharges => {
    logger.info(`UPDATING ACCUMULATED ACCOUNT CHARGES`);
    return Promise.map(updatedHourlyCharges, async charge => {
        logger.info(`Updating accumulated charge: ${JSON.stringify(charge)}`);
        const chargesPerAccount = convertToChargesPerAccount(charge);
        logger.info(`Charges per account: ${JSON.stringify(chargesPerAccount)}`);
        return Promise.map(chargesPerAccount, async accountCharge => {
            logger.info(`Updating account charge: ${JSON.stringify(accountCharge)}`);
            const accumulatedCharge = await getChargeFunc(accountCharge);
            if (!_.isEmpty(accumulatedCharge)) {
                const updatedAccumulatedCharge = {
                    ...accountCharge,
                    resources: _.concat(accumulatedCharge.resources, accountCharge.resources),
                    totalVal: accumulatedCharge.totalVal + accountCharge.totalVal
                };
                logger.info(`Updating new charge: ${JSON.stringify(updatedAccumulatedCharge)}`);
                return updateDatabaseFunc(updatedAccumulatedCharge);
            }
            logger.info(`Saving account charge`);
            return saveToDatabaseFunc(accountCharge);
        }, { concurrency: 1 })
            .catch(err => logger.info(`Failed to update accumulated table: ${err}`));
    }, { concurrency: 1 });
};

exports.totalPerAccount = totalPerAccount;
exports.totalPerSubscription = totalPerSubscription;
exports.convertToChargesPerAccount = convertToChargesPerAccount;
exports.resolveSubscription = resolveSubscription;
exports.saveHourlyCharge = saveHourlyCharge;
exports.updateHourlyChargeStatus = updateHourlyChargeStatus;
exports.reportMetering = reportMetering;
exports.processMeteringResponse = processMeteringResponse;
exports.resolveSubscriptions = resolveSubscriptions;
exports.saveToHourlyTable = saveToHourlyTable;
exports.updateHourlyTable = updateHourlyTable;
exports.updateAccumulatedCharges = updateAccumulatedCharges;
