const moment = require("moment");
const { resolveSubscriptions, saveToHourlyTable, totalPerAccount, totalPerSubscription, convertToChargesPerAccount, resolveSubscription, reportMetering, processMeteringResponse, updateHourlyTable } = require("../src/services/hourlyCharge");
const { validateAccumulatedChargePerAccountSchema, validateResolvedHourlyChargeRequest } = require("../src/services/validator");
const { processTimestamp } = require("../src/utils");
const requestBodyMock = require("./data-mocks/validChargingRequest");
const meteringDataMock = require("./data-mocks/meteringDataMock");
const resolveSubscriptionDataMock = require("./data-mocks/resolveSubscriptionDataMock");

describe.each([
    [requestBodyMock[0].accounts[0].resources, 200],
    [requestBodyMock[0].accounts[1].resources, 60],
    [requestBodyMock[1].accounts[0].resources, 13],
    [requestBodyMock[2].accounts[0].resources, 3],
    [requestBodyMock[3].accounts[0].resources, 0]
])(
    '.totalPerAccount(%o)',
    (resources, expected) => {
        test(`returns a sum resource's charges. Expected to return: ${expected}`, () => {
            expect(totalPerAccount(resources)).toBe(expected);
        });
    },
);

describe.each([
    [requestBodyMock[0].accounts, {
        accumulatedAccounts: [
            { ...requestBodyMock[0].accounts[0], totalVal: 200 },
            { ...requestBodyMock[0].accounts[1], totalVal: 60 }],
        accumulatedVal: 260
    }
    ],
    [requestBodyMock[1].accounts, {
        accumulatedAccounts: [{ ...requestBodyMock[1].accounts[0], totalVal: 13 }],
        accumulatedVal: 13
    }],
    [requestBodyMock[2].accounts, {
        accumulatedAccounts: [{ ...requestBodyMock[2].accounts[0], totalVal: 3 }],
        accumulatedVal: 3
    }],
    [requestBodyMock[3].accounts, {
        accumulatedAccounts: [{ ...requestBodyMock[3].accounts[0], totalVal: 0 }],
        accumulatedVal: 0
    }]
])(
    '.totalPerSubscription(%o)',
    (accounts, expected) => {
        test(`returns ${expected}`, () => {
            expect(totalPerSubscription(accounts)).toEqual(expected);
        });
    },
);

const subscriptionResponseMock = id => ({
    subscriptionStatus: 'subscribe-success',
    cloudProvider: id.split('-')[0],
    customerIdentifier: id.split('-')[2],
    productCode: id.split('-')[1],
    serviceName: 'cloud-sync',
    subscriptionId: id
});

describe('resolveSubscription', () => {
    test('given database get subscription function and subscription id, returns resolved promise with the database row associated with subscriptionId key', async () => {
        const myMock = jest.fn(subscriptionId => Promise.resolve(subscriptionResponseMock(subscriptionId)));
        await expect(resolveSubscription(myMock)('aws-eb85xigzhaaz2x4tho48267nf-B4tqtXBpknf')).resolves.toEqual({ value: subscriptionResponseMock('aws-eb85xigzhaaz2x4tho48267nf-B4tqtXBpknf'), status: "fulfilled" });
        expect(myMock.mock.calls.length).toBe(1);
    });
    test('returns resolved function even if database get function rejects', async () => {
        const myMock = jest.fn(subscriptionId => Promise.reject(new Error(`Failed to resolve ${subscriptionId}`)));
        await expect(resolveSubscription(myMock)('aws-eb85xigzhaaz2x4tho48267nf-B4tqtXBpknf')).resolves.toEqual({ error: new Error(`Failed to resolve aws-eb85xigzhaaz2x4tho48267nf-B4tqtXBpknf`), status: "rejected" });
        expect(myMock.mock.calls.length).toBe(1);
    });
});

describe('processTimestamp', () => {
    test('returns timestamp as a number', () => {
        expect(typeof processTimestamp(1556057542000)).toBe("number");
    });
    test('given epoch timestamp as number, returns epoch timestamp (UTC) as number of only the day (no milliseconds, seconds, minutes or hours) ', () => {
        // The timestamp is Tue Apr 23 2019 22:12:22 UTC and Wed Apr 24 2019 01:12:22 UTC+3. Function should return the 23rd and not the 24th
        expect(moment(processTimestamp(1556057542000)).toDate().getUTCDate()).toEqual(23);
        expect(moment(processTimestamp(1556057542000)).toDate().getUTCHours()).toEqual(0);
    });
    test('given epoch timestamp as number, and reduceTo "hourly", returns epoch timestamp (UTC) as number of only the day and hour (no milliseconds, seconds, minutes) ', () => {
        // The timestamp is Tue Apr 23 2019 22:12:22 UTC and Wed Apr 24 2019 01:12:22 UTC+3. Function should return the 23rd 10 PM
        expect(moment(processTimestamp(1556057542000, "hourly")).toDate().getUTCDate()).toEqual(23);
        expect(moment(processTimestamp(1556057542000, "hourly")).toDate().getUTCHours()).toEqual(22);
    });
});

describe('convertToChargesPerAccount', () => {
    test('given charge per subscription, returns a valid array of charges per account', async () => {
        const enhancedHourlyCharge = {
            mergedId: "aws-product1-customer1-dimension1",
            timestamp: "2019-05-01T09:54:36.945Z",
            subscriptionId: "aws-product1-customer1",
            customerIdentifier: "customer1",
            dimension: "dimension1",
            productCode: "product1",
            serviceName: "cloud-sync",
            cloudProvider: "aws",
            accounts: [
                {
                    accountId: "account-A",
                    resources: [
                        {
                            resourceId: "account-A-resource1",
                            val: 20
                        },
                        {
                            resourceId: "account-A-resource1",
                            val: 10
                        }
                    ],
                    totalVal: 30
                },
                {
                    accountId: "account-B",
                    resources: [
                        {
                            resourceId: "account-B-resource1",
                            val: 100
                        }
                    ],
                    totalVal: 100
                }
            ],
            accumulatedVal: 130,
            chargingStatus: true
        };
        const res = convertToChargesPerAccount(enhancedHourlyCharge);
        await expect(validateAccumulatedChargePerAccountSchema(res)).resolves.toBe(true);
    });
});

describe('processMeteringResponse', () => {
    test('given batch of enhanced hourly charges and successful metering response for that charge, returns enhanced hourly charge with appropriate chargingStatus', async () => {
        const enhancedHourlyChargeBatch = meteringDataMock.enhancedCharges;
        const meteringResponse = meteringDataMock.successfulMeteringResponse;
        const enhancedChargesWithStatus = processMeteringResponse(enhancedHourlyChargeBatch, meteringResponse);
        expect(enhancedChargesWithStatus[0].chargingStatus).toBe("success");
        expect(enhancedChargesWithStatus[1].chargingStatus).toBe("fail");
        expect(enhancedChargesWithStatus[2].chargingStatus).toBe("fail");
        expect(enhancedChargesWithStatus[3].chargingStatus).toBe("fail");
    });
});

describe('reportMetering', () => {
    test('given array of enhanced hourly charges and metering report function, returns the enhanced hourly charges with chargingStatus', async () => {
        const meteringReportMockFunc = jest.fn(() => Promise.resolve({
            Results: [
                {
                    MeteringRecordId: "some-id",
                    Status: "Success",
                    UsageRecord: {
                        CustomerIdentifier: "customer1",
                        Dimension: "dimension1",
                        Quantity: 30,
                        Timestamp: "2019-05-01T09:54:36.945Z"
                    }
                }
            ],
            UnprocessedRecords: []
        }));
        process.env.doCharge = "true";
        const enhancedChargesWithStatus = await reportMetering(meteringReportMockFunc)(meteringDataMock.differentProductEnhancedCharges);
        expect(enhancedChargesWithStatus.length).toBe(3);
        expect(enhancedChargesWithStatus[0].chargingStatus).toBe("success");
        expect(enhancedChargesWithStatus[1].chargingStatus).toBe("success");
        expect(enhancedChargesWithStatus[2].chargingStatus).toBe("fail");
        expect(meteringReportMockFunc.mock.calls.length).toBe(3);
    });
});

describe('resolveSubscriptions', () => {
    test('returns an enhanced hourly charge, in case the subscription was found in database', async () => {
        const subscriptionResolutionMockFunc = jest.fn(() => Promise.resolve({
            subscriptionId: "aws-product1-customer1",
            cloudProvider: "aws",
            customerIdentifier: "customer1",
            productCode: "product1",
            serviceName: "cvo",
            subscriptionStatus: "subscribed",
            createdAt: 1556637986696,
            accounts: { 0: "a" }
        }));
        const enhancedChargesWithStatus = await resolveSubscriptions(subscriptionResolutionMockFunc)(resolveSubscriptionDataMock);
        expect(enhancedChargesWithStatus[0].error).toEqual(undefined);
        expect(subscriptionResolutionMockFunc.mock.calls.length).toBe(1);
        await expect(validateResolvedHourlyChargeRequest(enhancedChargesWithStatus)).resolves.toBe(true);
    });
    test('returns an hourly charge with "subscription-notFound" error attribute, in case the subscription was not found in database', async () => {
        const subscriptionResolutionMockFunc = jest.fn(() => Promise.resolve({}));
        const enhancedChargesWithStatus = await resolveSubscriptions(subscriptionResolutionMockFunc)(resolveSubscriptionDataMock);
        expect(enhancedChargesWithStatus[0].error).toEqual("subscription-not-found");
        expect(subscriptionResolutionMockFunc.mock.calls.length).toBe(1);
        return validateResolvedHourlyChargeRequest(enhancedChargesWithStatus)
            .catch(err => {
                expect(err.details[0].message).toEqual(`"customerIdentifier" is required`);
            });
    });
    test('returns an hourly charge with subscription-status error, in case the subscription status is not subscribed or unsubscribed-pending', async () => {
        const subscriptionResolutionMockFunc = jest.fn(() => Promise.resolve({
            subscriptionId: "aws-product1-customer1",
            cloudProvider: "aws",
            customerIdentifier: "customer1",
            productCode: "product1",
            serviceName: "cvo",
            subscriptionStatus: "unsubscribed",
            createdAt: 1556637986696,
            accounts: { 0: "a" }
        }));
        const enhancedChargesWithStatus = await resolveSubscriptions(subscriptionResolutionMockFunc)(resolveSubscriptionDataMock);
        expect(enhancedChargesWithStatus[0].error).toEqual("subscription-status error: unsubscribed");
        expect(subscriptionResolutionMockFunc.mock.calls.length).toBe(1);
        return validateResolvedHourlyChargeRequest(enhancedChargesWithStatus)
            .catch(err => {
                expect(err.details[0].message).toEqual(`"customerIdentifier" is required`);
            });
    });
    test('returns an hourly charge with error, in case the resolve function rejects', async () => {
        const subscriptionResolutionMockFunc = jest.fn(() => Promise.reject(new Error("Some error occurred while resolving subscription")));
        const enhancedChargesWithStatus = await resolveSubscriptions(subscriptionResolutionMockFunc)(resolveSubscriptionDataMock);
        expect(enhancedChargesWithStatus[0].error.message).toEqual("Some error occurred while resolving subscription");
        expect(subscriptionResolutionMockFunc.mock.calls.length).toBe(1);
        return validateResolvedHourlyChargeRequest(enhancedChargesWithStatus)
            .catch(err => {
                expect(err.details[0].message).toEqual(`"customerIdentifier" is required`);
            });
    });
});

describe('saveToHourlyTable', () => {
    test('returns the same charge objects if successfully saved in table', async () => {
        const saveMockFunc = jest.fn(chrage => Promise.resolve(chrage));
        const meteringResponse = await saveToHourlyTable(saveMockFunc)(resolveSubscriptionDataMock);
        expect(meteringResponse).toEqual(resolveSubscriptionDataMock);
        expect(saveMockFunc.mock.calls.length).toBe(1);
    });
    test('returns the charge object enhanced with an error attribute if saving failed', async () => {
        const saveMockFunc = jest.fn(charge => Promise.reject(new Error(`Some error occurred with ${charge}`)));
        const meteringResponse = await saveToHourlyTable(saveMockFunc)(resolveSubscriptionDataMock);
        resolveSubscriptionDataMock[0].error = new Error(`Some error occurred with ${resolveSubscriptionDataMock[0]}`);
        expect(meteringResponse[0]).toEqual(resolveSubscriptionDataMock[0]);
        expect(saveMockFunc.mock.calls.length).toBe(1);
    });
});

describe('updateHourlyTable', () => {
    test('returns the same charge objects if successfully updated status in table', async () => {
        const updateMockFunc = jest.fn(() => Promise.resolve("Success"));
        const updateResponse = await updateHourlyTable(updateMockFunc)(resolveSubscriptionDataMock);
        expect(updateResponse).toEqual(resolveSubscriptionDataMock);
        expect(updateMockFunc.mock.calls.length).toBe(1);
    });
    test('returns the charge object enhanced with an error attribute if the original charge was not returned from database', async () => {
        const updateMockFunc = jest.fn(() => Promise.resolve({}));
        const updateResponse = await updateHourlyTable(updateMockFunc)(resolveSubscriptionDataMock);
        resolveSubscriptionDataMock[0].error = `Failed to update accumulated charge - original item not found!`;
        expect(updateResponse[0]).toEqual(resolveSubscriptionDataMock[0]);
        expect(updateMockFunc.mock.calls.length).toBe(1);
    });
    test('returns the charge object enhanced with an error attribute if updating failed', async () => {
        const updateMockFunc = jest.fn(charge => Promise.reject(new Error(`Some error occurred with ${charge}`)));
        const updateResponse = await updateHourlyTable(updateMockFunc)(resolveSubscriptionDataMock);
        resolveSubscriptionDataMock[0].error = new Error(`Some error occurred with ${resolveSubscriptionDataMock[0]}`);
        expect(updateResponse[0]).toEqual(resolveSubscriptionDataMock[0]);
        expect(updateMockFunc.mock.calls.length).toBe(1);
    });
});
