const chai = require("chai");
const expect = require('chai').expect;
const Constants = require('../utils/Constants');
const Initializer = require('../utils/specInitializer');
const _ = require('lodash');
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);

const { logger, clients } = Initializer('OCCM Integration Spec');

const resetAppMetadataAttributes = async () => {
    await clients.auth0Client.updateUserDetails(Constants.testimioUserAuth0Sub, {"app_metadata": {}});
    await clients.auth0Client.updateUserDetails(Constants.defaultUserAuth0Sub, {"app_metadata": {}});
};


describe('portal API Spec - getCustomerSuccessStories', function() {
    const enableNetappInternalScope = async (auth0UserId) => {
        await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:netapp-internal": true}});
    };
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetAppMetadataAttributes();
    });
    it('Should not return info if user is not netapp internal authorized', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const promise = clients.cloudCentralClient.getCustomerSuccessStories(userToken);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });

    it('Should return info if user is netapp internal authorized', async function() {
        await enableNetappInternalScope(Constants.testimioUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.testimioUserEmail, Constants.testimioUserPassword);
        const data = await clients.cloudCentralClient.getCustomerSuccessStories(userToken);
        logger.info(data);
        expect(data.includes('Company,Vertical,Region,Country,Use Case,Product,Cloud Provider,Public Reference,LinkName1,LinkUrl1,LinkName2,LinkUrl2,LinkName3,LinkUrl3,LinkName4,LinkUrl4,LinkName5,LinkUrl5,LinkName6,LinkUrl6,LinkName7,LinkUrl7,LinkName8,LinkUrl8,LinkName9,LinkUrl9')).to.equal(true);
        expect(data.slice(0,7)).to.equal("Company");
    });
});

describe('portal API Spec - uploadCustomerSuccessStories', function() {
    const enableCustomerStoriesScope = async (auth0UserId) => {
        await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:customer-stories": true}});
    };
    const enableNetappInternalScope = async (auth0UserId) => {
        await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:netapp-internal": true}});
    };
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetAppMetadataAttributes();
    });

    it('Should not allow uploading file if user is not netapp internal authorized', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const promise = clients.cloudCentralClient.uploadCustomerSuccessStories(userToken, "Company,Vertical,Region,Country,Use Case,Product,Cloud Provider,Public Reference");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });

    it('Should not allow uploading file that does not have the expected CSV columns headers', async function() {
        await enableCustomerStoriesScope(Constants.testimioUserAuth0Sub);
        await enableNetappInternalScope(Constants.testimioUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.testimioUserEmail, Constants.testimioUserPassword);
        const promise = clients.cloudCentralClient.uploadCustomerSuccessStories(userToken, "Company,Vertical,Region,Country,Use Case,Product,Cloud Provider,Public Reference");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('CSV file does not have expected column headers.');
                expect(error.response.status).to.equal(400);
            })
    });

    it('Should allow uploading correctly structured file by an authorized user', async function() {
        await enableCustomerStoriesScope(Constants.testimioUserAuth0Sub);
        await enableNetappInternalScope(Constants.testimioUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.testimioUserEmail, Constants.testimioUserPassword);
        let data = await clients.cloudCentralClient.getCustomerSuccessStories(userToken);
        let res = await clients.cloudCentralClient.uploadCustomerSuccessStories(userToken, data);
        expect(res).to.equal(204);
        data = await clients.cloudCentralClient.getCustomerSuccessStories(userToken);
        // noinspection BadExpressionStatementJS
        expect(data.slice(350,357)).to.not.be.empty;
    });

describe('portal API Spec - Update User Info ', function() {
        it('Should change user name info', async function() {
            const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
            const response = await clients.cloudCentralClient.updateUserInfo(userToken,'New Test Name');
            const nameRes = await clients.auth0Client.getUsersByQuery(`email:${Constants.defaultTestUserEmail}`,'user_metadata');
            logger.info(`${JSON.stringify(nameRes)}`);
            expect(nameRes[0].user_metadata.name).to.eql("New Test Name");
            expect(response).to.equal(204);
            logger.info("Return to originally name of Test user for forward checks");
            const response2 = await clients.cloudCentralClient.updateUserInfo(userToken,'Test User');
            expect(response2).to.equal(204);
        });
    });

describe('portal API Spec - Notifications ', function() {
        it('Should get user Notifications info ', async function() {
            const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
            const response = await clients.cloudCentralClient.getUserNotifications(userToken);
            logger.info(`${JSON.stringify(response)}`);
            const allNotificationsAreForCorrectUser = response.Items.filter(notification => notification.userId !== Constants.defaultUserAuth0Sub);
            expect(allNotificationsAreForCorrectUser.length).to.equal(0);
        });
    });
});
describe('portal API Spec - Users Id Info ', function() {
    const enableTenancyScope = async (auth0UserId) => {
        await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:tenancy": true}});
    };

    beforeEach(async function () {
        logger.debug('Inside before');
        await resetAppMetadataAttributes();
    });
    it('Should get users info - Happy path', async function() {
        await enableTenancyScope(Constants.defaultUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        logger.info(`${JSON.stringify(userToken)}`);
        const response = await clients.cloudCentralClient.getUsersIdInfo(userToken, Constants.defaultUserAuth0Sub);
        expect(response[0].userId).to.equal(Constants.defaultUserAuth0Sub);
        expect(response[0].email).to.equal(Constants.defaultTestUserEmail);

    });
    it('Calling get users Api with more then one Auth0 Ids', async function() {
        await enableTenancyScope(Constants.defaultUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const usersIds = Constants.defaultUserAuth0Sub + ',' + Constants.testimioUserAuth0Sub;
        const response = await clients.cloudCentralClient.getUsersIdInfo(userToken, usersIds);
        expect(response.length).to.equal(2);
        expect(response[0].userId).to.equal(Constants.defaultUserAuth0Sub);
        expect(response[0].email).to.equal(Constants.defaultTestUserEmail);
        expect(response[1].userId).to.equal(Constants.testimioUserAuth0Sub);
        expect(response[1].email).to.equal(Constants.testimioUserEmail);
    });
    it('Calling get users Api with more then one Auth0 Ids', async function() {
        await enableTenancyScope(Constants.defaultUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const fakeAuth0 = 'Auth0|bla-bla';
        const usersIds = Constants.defaultUserAuth0Sub + ',' + Constants.testimioUserAuth0Sub + ',' + fakeAuth0;
        const response = await clients.cloudCentralClient.getUsersIdInfo(userToken, usersIds);
        logger.info(`${JSON.stringify(response,null,4)}`);
        expect(response.length).to.equal(2);
        expect(response[0].userId).to.equal(Constants.defaultUserAuth0Sub);
        expect(response[0].email).to.equal(Constants.defaultTestUserEmail);
        expect(response[1].userId).to.equal(Constants.testimioUserAuth0Sub);
        expect(response[1].email).to.equal(Constants.testimioUserEmail);

    });
    it('Should get users info - With no cc:tenancy Scope should fail ', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const res = clients.cloudCentralClient.getUsersIdInfo(userToken, Constants.defaultUserAuth0Sub);
        return expect(res)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('portal API Spec - get user Id by email ', function() {
    const enableTenancyScope = async (auth0UserId) => {
        await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:tenancy": true}});
    };
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetAppMetadataAttributes();
    });
    it('Should get users info - Happy path', async function() {
        await enableTenancyScope(Constants.defaultUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        logger.info(`${JSON.stringify(userToken)}`);
        const response = await clients.cloudCentralClient.getUserIdByemail(userToken, Constants.defaultTestUserEmail);
        expect(response.userId).to.equal(Constants.defaultUserAuth0Sub);
    });
    it('Calling get user id Api with no existing email', async function() {
        await enableTenancyScope(Constants.defaultUserAuth0Sub);
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const fakeMail = 'blabla@netapp.com';
        const response = clients.cloudCentralClient.getUserIdByemail(userToken,fakeMail);
        return expect(response)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal(`No user has the given email address: ${fakeMail}`);
                expect(error.response.status).to.equal(400);

            })
    });
    it('With no cc:tenancy Scope should failed to get user id by email ', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const res = clients.cloudCentralClient.getUserIdByemail(userToken, Constants.defaultUserAuth0Sub);
        return expect(res)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('portal API Spec - Does user Exist in the portal ', function() {
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetAppMetadataAttributes();
    });
    it('Does user Exist - Happy path', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        logger.info(`${JSON.stringify(userToken)}`);
        const res1 = await clients.cloudCentralClient.doesUserExist(userToken, Constants.defaultTestUserEmail);
        expect(res1.doesExist).to.equal(true);
        const res2 = await clients.cloudCentralClient.doesUserExist(userToken, Constants.testimioUserEmail);
        expect(res2.doesExist).to.equal(true);
    });
    it('Does user Exist - No exist user should return false', async function() {
        const noExistUser = 'blabla@netapp.com';
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        logger.info(`${JSON.stringify(userToken)}`);
        const res1 = await clients.cloudCentralClient.doesUserExist(userToken, Constants.defaultTestUserEmail);
        expect(res1.doesExist).to.equal(true);
        const res2 = await clients.cloudCentralClient.doesUserExist(userToken, noExistUser);
        expect(res2.doesExist).to.equal(false);
    });
});

