const chai = require("chai");
const expect = require('chai').expect;
const Constants = require('../utils/Constants');
const Initializer = require('../utils/specInitializer');
const Promise = require('bluebird');
const jwtDecoder = require('../utils/general-utils').decodeToken;
const _ = require('lodash');
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);

const { logger, clients } = Initializer('OCCM Integration Spec');

const resetAppMetadataAttributes = async () => {
    await clients.auth0Client.updateUserDetails(Constants.testimioUserAuth0Sub, {"app_metadata": {}});
    await clients.auth0Client.updateUserDetails(Constants.defaultUserAuth0Sub, {"app_metadata": {}});
};

const resetUsersDynamoDevelopmentClients = async () => {
    await clients.dynamodbClient.updateUserDevelopmentClients(Constants.testimioUserAuth0Sub, []);
    await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, []);
};

const enableClientsScope = async (auth0UserId) => {
    await clients.auth0Client.updateUserDetails(auth0UserId, {"app_metadata": {"cc:clients": true}});
};

describe('Governance developer API Spec - getClients', function() {
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetUsersDynamoDevelopmentClients();
        await resetAppMetadataAttributes();
    });

    it('Should return no clients for a user that has no auth0 developmentUser permissions even if allow permission in dynamo', async function() {
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, ['*']);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const promise = clients.governanceClient.getClients(govToken);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                await clients.dynamodbClient.updateUserDevelopmentClients(Constants.testimioUserAuth0Sub, []);
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });

    it('Should fetch only the clients that a user is allowed to access in dynamo', async function() {
        await enableClientsScope(Constants.defaultUserAuth0Sub);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, [Constants.integrationClientId]);
        const clientsList = await clients.governanceClient.getClients(govToken);
        logger.debug(`Client list: ${JSON.stringify(clientsList)}`);
        expect(clientsList.clients.map(c => c.clientId)).to.eql([Constants.integrationClientId]);
        expect(clientsList.selectedClient.clientId).to.eql(Constants.integrationClientId);
    });
});

describe('Governance developer API Spec - getClient', function() {
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetUsersDynamoDevelopmentClients();
        await resetAppMetadataAttributes();
    });

    it('Should return no client details for a user that has no auth0 cc:clients scope even if allow permission in dynamo', async function() {
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, ['*']);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const promise = clients.governanceClient.getClient(govToken, Constants.integrationClientId);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });

    it('Should fetch the client details if a user is allowed to access in dynamo', async function() {
        await enableClientsScope(Constants.defaultUserAuth0Sub);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, [Constants.integrationClientId]);
        const clientDetails = await clients.governanceClient.getClient(govToken, Constants.integrationClientId);
        logger.debug(`Client info: ${JSON.stringify(clientDetails)}`);
        expect(clientDetails.clientId).to.equal(Constants.integrationClientId);
    });

    it('Should not fetch the client details if a user is not allowed access to it in dynamo', async function() {
        await enableClientsScope(Constants.defaultUserAuth0Sub);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, [Constants.integrationClientId]);
        const promise = clients.governanceClient.getClient(govToken, Constants.cloudCentralGovernanceClientId);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('Governance developer API Spec - updateClient', function() {
    beforeEach(async function () {
        logger.debug('Inside before');
        await resetUsersDynamoDevelopmentClients();
        await resetAppMetadataAttributes();
    });

    const URIs = () => {
        return {
            "allowedCallbackUrls": ["http://localhost:3001/"],
            "allowedWebOrigins": ["https://kishkush.com"],
            "allowedLogoutUrls": ["http://localhost:3001/"],
            "allowedCors": ["https://kishkush.com"]
        }
    };

    it('Should not allow to update client details if a user has no auth0 cc:clients scope, even if allow permission in dynamo', async function() {
        const fakeUria = {
            "allowedCallbackUrls": [],
            "allowedWebOrigins": [],
            "allowedLogoutUrls": [],
            "allowedCors": []
        };
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, ['*']);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const promise = clients.governanceClient.updateClient(govToken, Constants.integrationClientId, fakeUria);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });

    it('Should update client details that a user is allowed to access in dynamo', async function() {
        let uris = URIs();
        await enableClientsScope(Constants.defaultUserAuth0Sub);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        console.log(govToken);
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, [Constants.integrationClientId]);
        let updatedClientDetails = await clients.governanceClient.updateClient(govToken, Constants.integrationClientId, uris);
        logger.debug(`Client info: ${JSON.stringify(updatedClientDetails)}`);
        expect(updatedClientDetails.clientId).to.equal(Constants.integrationClientId);
        expect(updatedClientDetails.allowedCallbackUrls).to.eql(uris.allowedCallbackUrls);
        expect(updatedClientDetails.allowedWebOrigins).to.eql(uris.allowedWebOrigins);
        expect(updatedClientDetails.allowedLogoutUrls).to.eql(uris.allowedLogoutUrls);
        expect(updatedClientDetails.allowedCors).to.eql(uris.allowedCors);
        uris.allowedWebOrigins.pop();
        uris.allowedCors.pop();
        updatedClientDetails = await clients.governanceClient.updateClient(govToken, Constants.integrationClientId, uris);
        logger.debug(`Client info: ${JSON.stringify(updatedClientDetails)}`);
        expect(updatedClientDetails.allowedCors).to.eql([]);
        expect(updatedClientDetails.allowedWebOrigins).to.eql([]);

    });

    it('Should not update client details if a user is not allowed to access it in dynamo', async function() {
        let uris = URIs();
        await enableClientsScope(Constants.defaultUserAuth0Sub);
        const govToken = await clients.auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        await clients.dynamodbClient.updateUserDevelopmentClients(Constants.defaultUserAuth0Sub, [Constants.cloudCentralGovernanceClientId]);
        const promise = clients.governanceClient.updateClient(govToken, Constants.integrationClientId, uris);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(async error => {
                expect(error.response.data.message).to.equal('Forbidden');
                expect(error.response.status).to.equal(403);
            })
    });
});

