const chai = require("chai");
const assert = require('chai').assert;
const expect = require('chai').expect;
const Constants = require('../utils/Constants');
const Initializer = require('../utils/specInitializer');
const Promise = require('bluebird');
const jwtDecoder = require('../utils/general-utils').decodeToken;
const _ = require('lodash');
const SystemIDuuv = require('uuid/v4');
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);

const { logger, clients } = Initializer('OCCM Integration Spec');

const publicIp = '1.1.1.1';
const privateIp = '2.2.2.2';
const systemId = "test-system-id";
const occmName = "test-name";
const redirectURIs = [
    `http://${publicIp}`,
    `http://${publicIp}/occmui`,
    `https://${publicIp}`,
    `https://${publicIp}/occmui`,
    `http://${privateIp}`,
    `http://${privateIp}/occmui`,
    `https://${privateIp}`,
    `https://${privateIp}/occmui`
];

const waitForDynamoToUpdate = async (millisec) => {
    await Promise.delay(millisec);
};

describe('OCCM subscription flow', function() {
    it('Should allow an existing user to subscribe an OCCM via set up as a service', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const { clientId, clientSecret } = await clients.occmClient.serviceRegisterOccm(systemId, occmName, userToken);

        logger.debug(`New OCCM registered in auth0 with clientId: ${clientId} and clientSecrete: ${clientSecret}`);

        await waitForDynamoToUpdate(500);

        let auth0ApplicationInfo = await clients.auth0Client.getClientById(clientId);
        logger.debug(`OCCM is registered in auth0 with details: ${JSON.stringify(auth0ApplicationInfo)}`);
        verifyAuth0AfterServiceRegistration(auth0ApplicationInfo);

        const dynamoUserEntry = await clients.dynamodbClient.getUserInfo(jwtDecoder(userToken).sub);
        logger.debug(`User's info in dynamoDb: ${JSON.stringify(dynamoUserEntry)}`);
        let dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        verifyDynamoAfterServiceRegistration(dynamoOccmEntry, clientId, systemId, occmName, jwtDecoder(userToken).sub, dynamoUserEntry);

        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        const updateUriResponse = await clients.occmClient.updateRedirectUris(redirectURIs, occmToken);
        expect(updateUriResponse).to.equal(204);

        await waitForDynamoToUpdate(500);

        auth0ApplicationInfo = await clients.auth0Client.getClientById(clientId);
        logger.debug(`OCCM is registered in auth0 with details: ${JSON.stringify(auth0ApplicationInfo)}`);
        verifyAuth0AfterServiceUpdateUris(auth0ApplicationInfo);

        dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`User's info in dynamoDb: ${JSON.stringify(dynamoUserEntry)}`);
        verifyDynamoAfterServiceUpdateUris(dynamoOccmEntry, redirectURIs);

        const clientGrants = await clients.auth0Client.getClientGrants(clientId);
        expect(clientGrants[0].scope.length).is.equal(1);
        expect(clientGrants[0].scope[0]).is.equal("occm-access");

        await deleteUserOccm(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        });

    it('Should allow an existing user to subscribe an OCCM via marketplace without allowing registration duplication', async function() {
        const systemId = SystemIDuuv();

        const registratorToken = await clients.auth0Client.getOccmRegistratorToken();
        const {clientId, clientSecret, alreadyExists, setupComplete} = await clients.occmClient.registerOccm(redirectURIs, systemId, registratorToken);
        logger.debug(`New OCCM registered in auth0 with clientId: ${clientId} and clientSecrete: ${clientSecret}`);
        expect(alreadyExists).to.equal(false);
        expect(setupComplete).to.equal(false);

        logger.info(`Re-registering OCCM with systemId ${systemId}`);
        const { clientId: clientId2, clientSecret: clientSecret2, alreadyExists: alreadyExists2,  setupComplete: setupComplete2} = await clients.occmClient.registerOccm(redirectURIs, systemId, registratorToken);
        expect(clientId2).to.equal(clientId);
        expect(clientSecret2).to.equal(clientSecret);
        expect(alreadyExists2).to.equal(true);
        expect(setupComplete2).to.equal(false);

        await waitForDynamoToUpdate(500);

        const auth0ApplicationInfo = await clients.auth0Client.getClientById(clientId);
        logger.debug(`OCCM is registered in auth0 with details: ${JSON.stringify(auth0ApplicationInfo)}`);
        verifyAuth0AfterMarketplaceRegistration(auth0ApplicationInfo);

        let dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        verifyDynamoAfterMarketplaceRegistration(dynamoOccmEntry, clientId, systemId, redirectURIs);

        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        await clients.occmClient.setupComplete(systemId, occmToken);
        const { setupComplete: setupComplete3} = await clients.occmClient.registerOccm(redirectURIs, systemId, registratorToken);
        expect(setupComplete3).to.equal(true);

        dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        verifyDynamoAfterMarketplaceSetupComplete(dynamoOccmEntry, systemId, jwtDecoder(occmToken).sub);

        await cleanAuth0AndDynamo(occmToken, clientId);
    });

    it('Should return 400 when updating badly formed URIs', async function() {
        const incorrectRedirectURIs = [`http:// ${publicIp}`]; //Intentionally incorrect with space.

        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const { clientId, clientSecret } = await clients.occmClient.serviceRegisterOccm(systemId, occmName, userToken);
        logger.debug(`New OCCM registered in auth0 with clientId: ${clientId} and clientSecrete: ${clientSecret}`);

        await Promise.delay(500);

        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        const promise = clients.occmClient.updateRedirectUris(incorrectRedirectURIs, occmToken);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message).to.equal('StatusCode: 400 Message: Payload validation error: \'Object didn\'t pass validation for format url: http:// 1.1.1.1\' on property allowed_logout_urls[0].');
                expect(error.response.status).to.equal(400);
                return deleteUserOccm(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
            })
    });

    it('Should turn on setup failure flag in dynamo in case of setup issue', async function() {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const { clientId, clientSecret } = await clients.occmClient.serviceRegisterOccm(systemId, occmName, userToken);
        logger.debug(`New OCCM registered in auth0 with clientId: ${clientId} and clientSecrete: ${clientSecret}`);

        await waitForDynamoToUpdate(500);

        const response = await clients.occmClient.serviceSetupFailed(userToken, clientId);
        expect(response).to.equal(204);

        let dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        expect(dynamoOccmEntry.Item.setupFailed).to.equal(true);

        await deleteUserOccm(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
    });

    it('OCCM agent registration should create OCCM with agent===true and client grant agents:agent', async function () {
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const serviceRegisterResponse = await clients.occmClient.serviceRegisterOccm(systemId,occmName,userToken,true);
        logger.debug(`After Service registration : ${JSON.stringify(serviceRegisterResponse)}`);
        const auth0ClientGrantRespond = await clients.auth0Client.getClientGrants(serviceRegisterResponse.clientId);
        logger.debug(`Auth0 response after client grants : ${JSON.stringify(auth0ClientGrantRespond)}`);
        expect(auth0ClientGrantRespond[0].scope[1]).to.equal("agents:agent");
        let dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(serviceRegisterResponse.clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        expect(dynamoOccmEntry.Item.agent).to.equal(true);
        logger.info(JSON.stringify(auth0ClientGrantRespond[0].scope[1]));
        await deleteUserOccm(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
    });

});

describe('OCCM user management', function() {
    let clientId, clientSecret;

    before(async function () {
        logger.debug('Inside before');
        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        ({clientId, clientSecret} = await clients.occmClient.serviceRegisterOccm(systemId, occmName, userToken));
        logger.debug(`New OCCM registered in auth0 with clientId: ${clientId} and clientSecrete: ${clientSecret}`);
        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        await clients.occmClient.updateRedirectUris(redirectURIs, occmToken);
        await deleteUserOccm(Constants.testimioUserEmail, Constants.testimioUserPassword);
    });

    after(async function () {
        logger.debug('Inside after');
        await deleteUserOccm(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        await deleteUserOccm(Constants.testimioUserEmail, Constants.testimioUserPassword);
    });

    it('Should allow associating new user with OCCM and deassociate existing users', async function() {
        const associateeEmail = Constants.testimioUserEmail;

        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        let associateUserResponse = await clients.occmClient.associateUser(occmToken, associateeEmail, Constants.testimioUserAuth0Sub);
        expect(associateUserResponse).to.eql({userId:Constants.testimioUserAuth0Sub});
        logger.info(`re-associating same user again expecting no error`);
        associateUserResponse = await clients.occmClient.associateUser(occmToken, associateeEmail, Constants.testimioUserAuth0Sub);
        console.log(`SECOND RES ${JSON.stringify(associateUserResponse)}`);
        expect(associateUserResponse).to.eql({userId:Constants.testimioUserAuth0Sub});

        const associateeToken = await clients.auth0Client.getCloudCentralUserToken(associateeEmail, Constants.testimioUserPassword);
        const { occms, hadPreviousOccm } = await clients.occmClient.listUserOccms(associateeToken);
        logger.debug(`user ${associateeEmail} occms after association: ${JSON.stringify(occms)}`);
        expect(occms.length).to.equal(1);
        expect(hadPreviousOccm).to.equal(true);
        expect(occms[0].occm).to.equal(clientId);

        const occmUserResponse = await clients.occmClient.listOccmUsers(occmToken, `${Constants.defaultUserAuth0Sub},${jwtDecoder(associateeToken).sub}`);
        logger.debug(`OCCM's users after association: ${JSON.stringify(occmUserResponse)}`);
        expect(occmUserResponse.length).to.equal(2);
        expect(occmUserResponse[0].userId).to.equal(jwtDecoder(associateeToken).sub);
        expect(occmUserResponse[1].userId).to.equal(Constants.defaultUserAuth0Sub);
        const resDeass = await clients.occmClient.deassociateUser(occmToken, Constants.defaultUserAuth0Sub, jwtDecoder(associateeToken).sub);
        expect(resDeass).to.equal(204);
        const { occms: occms2} = await clients.occmClient.listUserOccms(associateeToken);
        logger.debug(`user ${associateeEmail} occms: ${JSON.stringify(occms2)}`);
        expect(occms2.length).to.equal(0);
    });

    it('Should throw error when associating non existing user', async function() {
        const associateeEmail = "notexistinguser@netapp-test.com";

        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        await clients.occmClient.updateRedirectUris(redirectURIs, occmToken);

        const promise = clients.occmClient.associateUser(occmToken, associateeEmail, Constants.defaultUserAuth0Sub);
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message).to.equal(`No user has the given email address: ${associateeEmail}, the user needs to first sign up to Cloud Central (cloud.netapp.com).`);
                expect(error.response.status).to.equal(400);
            });
    });

    it('Should allow user to see and manually update OCCM\'s URIs', async function() {
        const newRedirectURIs = [
            {isDefault: true, value: 'http://3.3.3.3'},
            {isDefault: false, value: 'http://4.4.4.4'}
        ];

        const userToken = await clients.auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);

        const currentUris = await clients.occmClient.listUris(occmToken);
        expect(currentUris).to.eql(redirectURIs);

        const response = await clients.occmClient.updateRedirectUrisManually(userToken, clientId, newRedirectURIs);
        expect(response).to.equal(204);

        await waitForDynamoToUpdate(500);

        const auth0ApplicationInfo = await clients.auth0Client.getClientById(clientId);
        logger.debug(`OCCM is registered in auth0 with details: ${JSON.stringify(auth0ApplicationInfo)}`);
        expect(auth0ApplicationInfo.callbacks.length).to.equal(24);
        expect(auth0ApplicationInfo.allowed_logout_urls.length).to.equal(10);

        let dynamoOccmEntry = await clients.dynamodbClient.getClientInfo(clientId);
        logger.debug(`OCCM's info in dynamoDb: ${JSON.stringify(dynamoOccmEntry)}`);
        expect(dynamoOccmEntry.Item.manualOverrideUris).to.eql(newRedirectURIs);
    });

    it('Should not except strings as objects', async function () {
        const occmToken = await clients.auth0Client.getOccmToken(clientId, clientSecret);
        const associateeEmail = "some.email@netapp-test.com";

        let associateUserPromise = clients.occmClient.associateUserWithRawData(occmToken, JSON.stringify(JSON.stringify({
            associateeEmail,
            associaterId: Constants.defaultUserAuth0Sub
        })));

        return expect(associateUserPromise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message).to.equal(`Body failed validation. "value" must be an object`);
                expect(error.response.status).to.equal(400);
            });
    });
});

const cleanAuth0AndDynamo = (userToken, clientId) => {
    return Promise.all([
        clients.occmClient.deassociateSelf(userToken, clientId),
        clients.auth0Client.deleteClient(clientId),
        clients.dynamodbClient.deleteOccm(clientId)
    ]);
};

const deleteUserOccm = async (email, password) => {
    const govUserToken = await clients.auth0Client.getGovernanceUserToken(email, password);
    await clients.governanceClient.deleteAssociatedOccms(govUserToken);
};

const verifyDynamoAfterServiceRegistration = (dynamoOccmResponse, clientId, systemId, occmName, auth0UserId, dynamoUserResponse) => {
    expect(dynamoOccmResponse.Item.clientName).to.equal(`Occm-Client-${systemId}`);
    expect(dynamoOccmResponse.Item.saas).to.equal(true);
    expect(dynamoOccmResponse.Item.occmName).to.equal(occmName);
    expect(dynamoOccmResponse.Item.clientId).to.equal(clientId);
    expect(dynamoOccmResponse.Item.setupUserId).to.equal(auth0UserId);
    expect(dynamoOccmResponse.Item.agent).to.equal(false);
    expect(_.includes(dynamoUserResponse.Item.associatedOccms.values, clientId)).to.equal(true);
};

const verifyDynamoAfterServiceUpdateUris = (dynamoOccmResponse, redirectURIs) => {
    expect(dynamoOccmResponse.Item.automaticCallbackUris).to.eql(redirectURIs);
    expect(dynamoOccmResponse.Item.setupComplete).to.eql(true);
};

const verifyAuth0AfterServiceRegistration = (auth0Response) => {
    expect(auth0Response.allowed_origins.length).to.equal(5);
    assert.isUndefined(auth0Response.callbacks);
    assert.isUndefined(auth0Response.allowed_logout_urls);
};

const verifyAuth0AfterServiceUpdateUris = (auth0Response) => {
    expect(auth0Response.allowed_origins.length).to.equal(5);
    expect(auth0Response.callbacks.length).to.equal(16);
    expect(auth0Response.allowed_logout_urls.length).to.equal(8);
};

const verifyAuth0AfterMarketplaceRegistration = (auth0Response) => {
    assert.isUndefined(auth0Response.allowed_origins);
    expect(auth0Response.callbacks.length).to.equal(16);
    expect(auth0Response.allowed_logout_urls.length).to.equal(8);
};

const verifyDynamoAfterMarketplaceRegistration = (dynamoOccmResponse, clientId, systemId, redirectURIs) => {
    expect(dynamoOccmResponse.Item.clientName).to.equal(`Occm-Client-${systemId}`);
    assert.isUndefined(dynamoOccmResponse.Item.saas);
    expect(dynamoOccmResponse.Item.clientId).to.equal(clientId);
    expect(dynamoOccmResponse.Item.automaticCallbackUris.length).to.equal(8);
    expect(dynamoOccmResponse.Item.automaticCallbackUris).to.eql(redirectURIs);
};

const verifyDynamoAfterMarketplaceSetupComplete = (dynamoOccmResponse, systemId, registratorClientId) => {
    expect(dynamoOccmResponse.Item.setupComplete).to.equal(true);
    expect(dynamoOccmResponse.Item.occmName).to.equal(`OccmName-${systemId}`);
    expect(dynamoOccmResponse.Item.setupUserId).to.equal(registratorClientId);
};

