// const DynamoDb = require('aws-sdk/clients/DynamoDB');
const Constants = require('../Constants');
const logger = require('../../logger/Logger');
const AWS = require('aws-sdk');
AWS.config.setPromisesDependency(require('bluebird'));

class DynamoDbClient {

    constructor(logger, awsOptions = {}) {
        this.dynamoDb = new AWS.DynamoDB.DocumentClient();
        this.logger = logger;
    }

    async getClientInfo(clientId) {
        logger.info(`Getting ${clientId} info from ${Constants.occmClientInfoTable} table`);
        const params = {
            TableName: Constants.occmClientInfoTable,
            Key: { clientId }
        };
        try {
            const response = await this.dynamoDb.get(params).promise();
            return response || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }

    async getUserInfo(auth0UserId) {
        logger.info(`Getting ${auth0UserId} info from ${Constants.occmUserInfoTable} table`);
        const params = {
            TableName: Constants.occmUserInfoTable,
            Key: {userId: auth0UserId}
        };
        try {
            const response = await this.dynamoDb.get(params).promise();
            return response || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }

    async deleteOccm(clientId) {
        logger.info(`Deleting ${clientId} info from ${Constants.occmClientInfoTable} table`);
        const params = {
            TableName: Constants.occmClientInfoTable,
            Key: { clientId }
        };
        try {
            const response = await this.dynamoDb.delete(params).promise();
            return response || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }

    async updateUserDevelopmentClients(userId, clients) {
        // If client list is empty, developmentClients attribute is removed completely
        logger.info(`Updating ${userId} developmentClients info to include: ${clients}, in ${Constants.occmUserInfoTable} table`);
        const params = {
            TableName : Constants.occmUserInfoTable,
            Key: { userId },
            UpdateExpression: clients.length === 0 ? "REMOVE developmentClients" : "set developmentClients= :arrp",
            ReturnValues:"UPDATED_NEW"
        };
        if(clients.length !== 0) {
            params.ExpressionAttributeValues = {":arrp": this.dynamoDb.createSet(clients)}
        }
        try {
            const response = await this.dynamoDb.update(params).promise();
            return response || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }

    async updateSubordinateUsers(userId, subordinateUsers) {
        // If client subordinateUsers is empty, subordinateUsers attribute is removed completely
        logger.info(`Updating ${userId} subordinateUsers info to include: ${subordinateUsers}, in ${Constants.occmUserInfoTable} table`);
        const params = {
            TableName : Constants.occmUserInfoTable,
            Key: { userId },
            UpdateExpression: subordinateUsers.length === 0 ? "REMOVE subordinateUsers" : "set subordinateUsers= :arrp",
            ReturnValues:"UPDATED_NEW"
        };
        if(subordinateUsers.length !== 0) {
            params.ExpressionAttributeValues = {":arrp": this.dynamoDb.createSet(subordinateUsers)}
        }
        try {
            const response = await this.dynamoDb.update(params).promise();
            return response || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }

    async getStaticClientsWithDisplayName() {
        logger.info(`Getting static clients with display name from ${Constants.staticClients} table`);
        const params = {
            TableName : Constants.staticClients,
            FilterExpression: 'attribute_exists(displayName)',
            Limit: 100
        };
        try {
            const response = await this.dynamoDb.scan(params).promise();
            return response.Items || null;
        } catch (e) {
            logger.error(e);
            throw e;
        }
    }
}

module.exports = DynamoDbClient;




