import React from 'react';
import { connect } from 'react-redux';
import _ from 'lodash';

import { ReactComponent as UserIcon } from "ui-components/src/images/monochrome-icons/ic-user.svg";
import { ReactComponent as AgentIcon } from "ui-components/src/images/monochrome-icons/ic-agent.svg";
import buttons from "ui-components/src/button/button.module.scss";
import styles from "./Accounts.module.scss";
import Loader from "../../Loader";

const isWorkspacesEqual = (workspace1, workspace2) => workspace1 && workspace2 && workspace1.workspacePublicId && workspace2.workspacePublicId && workspace1.workspacePublicId === workspace2.workspacePublicId;

class ManageWorkspacesOverlay extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            associatedWorkspaces: this.props.entityForWorkspacesManagement.workspaces.map(workspace => _.assignIn({}, workspace, { checked: true })),
            unAssociatedWorkspaces: _.differenceWith(this.props.accountWorkspaces, this.props.entityForWorkspacesManagement.workspaces, isWorkspacesEqual),
            changedDone: false
        };
    }

    updateChangesDone() {
        this.setState({
            changesDone: _.some(this.state.associatedWorkspaces, workspace => !workspace.checked) ||
                _.some(this.state.unAssociatedWorkspaces, 'checked')
        });
    }

    handleCheckboxChange = (workspacesGroup, workspacePublicId) => {
        this.setState(prevState => ({
            [workspacesGroup]: prevState[workspacesGroup].map(workspace => {
                if (workspace.workspacePublicId === workspacePublicId) workspace.checked = !workspace.checked;
                return workspace;
            })
        }), this.updateChangesDone);
    };

    submitButtonClicked() {
        const workspaceIdsToRemove = _(this.state.associatedWorkspaces).reject('checked').map('workspacePublicId').value();
        const workspaceIdsToAdd = _(this.state.unAssociatedWorkspaces).filter('checked').map('workspacePublicId').value();

        this.props.manageEntityWorkspaces({
            entity: this.props.entityForWorkspacesManagement,
            workspaceIdsToRemove,
            workspaceIdsToAdd
        });
    }

    render() {
        return (<div className={styles["manage-workspaces-overlay"]}>
            {this.props.manageEntityWorkspacesPending && <Loader/>}
            {this.props.type === 'User' && <UserIcon/>}
            {this.props.type === 'Agent' && <AgentIcon/>}
            <div className={styles["icon-title"]}>{this.props.entityForWorkspacesManagement.name}</div>
            <div className={styles["title"]}>Manage the {this.props.type}'s Workspaces</div>
            <div className={styles["workspaces-list"]}>
                {(this.state.associatedWorkspaces.length || null) && <h5>Associated Workspaces:</h5>}
                {this.state.associatedWorkspaces.map(workspace => (
                    <label key={workspace.workspacePublicId} title={workspace.workspaceName} style={{ fontWeight: 600 }}>
                        <input type="checkbox"
                               defaultChecked={true}
                               onChange={() => this.handleCheckboxChange('associatedWorkspaces', workspace.workspacePublicId)}/>
                        <span>{workspace.workspaceName}</span>
                    </label>))}
                {(this.state.unAssociatedWorkspaces.length || null) && (this.state.associatedWorkspaces.length || null) && <h5>Other Workspaces:</h5>}
                {this.state.unAssociatedWorkspaces.map(workspace => (
                    <label key={workspace.workspacePublicId} title={workspace.workspaceName}>
                        <input type="checkbox" defaultChecked={false}
                               onChange={() => this.handleCheckboxChange('unAssociatedWorkspaces', workspace.workspacePublicId)}/>
                        <span>{workspace.workspaceName}</span>
                    </label>))}
            </div>
            <div className={styles["button-box"]}>
                <button type="button" className={buttons.cancel} onClick={() => {
                    this.props.manageEntityWorkspacesPrompt({});
                }}>Cancel
                </button>
                <button type="submit" className={buttons.primary}
                        disabled={!this.state.changesDone || this.props.manageEntityWorkspacesPending}
                        onClick={this.submitButtonClicked.bind(this)}>Apply
                </button>
            </div>
            {this.props.manageEntityWorkspacesError && <div className={styles["error-message"]}>{this.props.manageEntityWorkspacesError}</div>}
        </div>);
    }
}

const mapStateToPropsForUser = state => ({
    accountWorkspaces: state.accountsManagement.accountWorkspaces,
    entityForWorkspacesManagement: state.accountsManagement.userForWorkspacesManagement,
    manageEntityWorkspacesPending: state.accountsManagement.manageUserWorkspacesPending,
    manageEntityWorkspacesError: state.accountsManagement.manageUserWorkspacesError,
    type: "User"
});

const mapDispatchToPropsForUser = dispatch => ({
        manageEntityWorkspacesPrompt: ({ entity }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PROMPT",
                payload: entity
            });
        },
        manageEntityWorkspaces: ({ entity, workspaceIdsToRemove, workspaceIdsToAdd }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES",
                payload: { user: entity, workspaceIdsToRemove, workspaceIdsToAdd }
            });
        }
    }
);

const mapStateToPropsForAgent = state => ({
    accountWorkspaces: state.accountsManagement.accountWorkspaces,
    entityForWorkspacesManagement: state.accountsManagement.agentForWorkspacesManagement,
    manageEntityWorkspacesPending: state.accountsManagement.manageAgentWorkspacesPending,
    manageEntityWorkspacesError: state.accountsManagement.manageAgentWorkspacesError,
    type: "Agent"
});

const mapDispatchToPropsForAgent = dispatch => ({
        manageEntityWorkspacesPrompt: ({ entity }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-PROMPT",
                payload: { agent: entity }
            });
        },
        manageEntityWorkspaces: ({ entity, workspaceIdsToRemove, workspaceIdsToAdd }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES",
                payload: { agent: entity, workspaceIdsToRemove, workspaceIdsToAdd }
            });
        }
    }
);

const ManageUserWorkspacesOverlay = connect(mapStateToPropsForUser, mapDispatchToPropsForUser)(ManageWorkspacesOverlay);
const ManageAgentWorkspacesOverlay = connect(mapStateToPropsForAgent, mapDispatchToPropsForAgent)(ManageWorkspacesOverlay);

export { ManageUserWorkspacesOverlay, ManageAgentWorkspacesOverlay };
