import React from 'react'
import CloudAssessmentSecuredInBrowserBox from "./CloudAssessmentSecuredInBrowserBox"
import ActionButton from "./../widgets/ActionButton";
import { ReactComponent as AzureLogo } from 'ui-components/src/images/third-party-services/logo-azure.svg';
import { openDialog } from "./../dialog/Dialog"
import NeedHelpContactUsWidget from "./NeedHelpContactUsWidget"
import { ReactComponent as DialogCloseXIcon } from 'ui-components/src/images/monochrome-icons/x-icon.svg';
import { Scrollbars } from 'react-custom-scrollbars';
import { connect } from 'react-redux';
import querystring from 'query-string';
import BlankPage from './../BlankPage'
import CircleSpinner from "./../widgets/CircleSpinner";
import { CopyToClipboard } from 'react-copy-to-clipboard';
import {SyntaxHighlighter} from "ui-components"
import ExternalLink from "./../widgets/ExternalLink";
import scrollToComponent from 'react-scroll-to-component'
import styles from "./CloudAssessmentView.module.scss";
import FormFieldSelect from "./../widgets/FormFieldSelect"

const policy = `
{
    "Name": "NetApp Azure Assessment Tool User",
    "Actions": [
        "Microsoft.Resources/subscriptions/read",
        "Microsoft.Compute/disks/read",
        "Microsoft.Compute/snapshots/read",
        "Microsoft.Compute/virtualMachines/read"       
    ],
    "NotActions": [],
    "AssignableScopes": [
        "/subscriptions/***FILL IN YOUR SUBSCRIPTION ID***"
    ],
    "Description": "Azure use assessment tool",
    "IsCustom": "true"
}
`;

const PolicyRequirementsDialog = ({closeDialog}) =>
    <div>
        <header>
            Policy Requirements
            <button className="transparent" onClick={closeDialog}><DialogCloseXIcon/></button>
        </header>
        <section>
            <Scrollbars
                thumbSize={126}
                style={{ width: "100%", height: "100%"}}
                renderThumbVertical={props => {
                    const style = props.style;
                    return <div {...props} style={{...style, cursor: "pointer", width: "5px", right: 5, borderRadius: "5px", backgroundColor: "#e9e9e9"}} />
                }}>
                <div className="inner">
                    <p>Cloud Assessment requires use of an Azure enterprise account.</p>
                    <p>Logging in with Azure will request consent to grant us permissions to perform the assessment on your behalf. The logged in user should have the correct IAM permissions to perform the assessment.</p>
                    <p>For more information about IAM Access control, refer to the following Azure documentation: <ExternalLink linkKey="cloudAssessment.credentials.azureRolesDocumentation" newTab>RBAC and the Azure portal</ExternalLink></p>
                    <p>The logged in user should have a IAM role with the following permissions:</p>
                    <div style={{position: "relative"}}>
                        <CopyToClipboard text={policy} style={{padding: "10px 20px", position: "absolute", top: 0, right: 0, borderRadius: 4}}><ActionButton category="CloudAssessment" label="CopyPolicyToClipboard">Copy to clipboard</ActionButton></CopyToClipboard>
                        <SyntaxHighlighter language="json" customStyle={{background: "rgb(246, 246, 246)", borderRadius: 4}}>{policy}</SyntaxHighlighter>
                    </div>
                </div>
            </Scrollbars>

        </section>
    </div>
;

const Logout = ({logout}) => {
  return <div className="logout"><button className="a bolder" onClick={logout}>Login with a different account</button></div>
};

const TokenFailure = ({errorMessage, tryAgain}) =>
    <div className="error">
        <p>Could not complete login. This could be because the login window was closed before completion, or consent to log in was denied. <br/> {errorMessage}</p>
        <button className="login" onClick={tryAgain}>Try Again</button>
    </div>
;

const FetchingSubscriptions = () => <CircleSpinner/>;

const SubscriptionFailure = ({errorMessage}) => <div className="error">
    <p>Failed to fetch subscriptions.  <br/> {errorMessage}</p>
</div>;

const NoSubscriptions = () => <div className="error">
    <p>There are no subscriptions in your account, or you do not have permissions to view any subscriptions. <br/> Unable to perform assessment.</p>
</div>;


let errorEl = null;
const triggerError = (el) => {
    if(el !== errorEl){
        errorEl = el;
        if(el) {
            scrollToComponent(errorEl, { offset: -200, align: 'top', duration: 400});
        }
    }
};

const CredentialsStep = ({stage, performAssessment, selectedSubscription, selectSubscription, azureSubscriptions, login, logout, errorMessage, azureUserEmail}) => {
    if(querystring.parse(window.location.search).relogin !== undefined){
        login();
        return <BlankPage/>
    } else {
        return <div className="login-container credentials-step">
            <CloudAssessmentSecuredInBrowserBox/>
            <div className="cloud-provider-logo"><AzureLogo/></div>

            {stage === "CREDENTIALS" && <div>
                <p style={{marginBottom: 5}}>Please read the Azure <ActionButton className="a" category="CloudAssessment" label="PoliciesDialog" onClick={(el) => {
                    openDialog(<PolicyRequirementsDialog className="cloud-assessment-keys-dialog"/>, el.target)
                }}>Policy Requirements</ActionButton> before you continue</p>
                <p>You will be redirected to Azure and must login with your Azure enterprise account in order to proceed.</p>

                <button className="login" onClick={login}>Login with Azure</button>
            </div>}
            {stage === "CREDENTIALS-AZURE-ACCESS-TOKEN-FAILURE" && <TokenFailure errorMessage={errorMessage} tryAgain={login}/>}
            {stage === "CREDENTIALS-FETCHING-SUBSCRIPTIONS" && <FetchingSubscriptions/>}
            {stage === "CREDENTIALS-FETCH-SUBSCRIPTIONS-FAILED" && <SubscriptionFailure errorMessage={errorMessage}/>}
            {stage === "CREDENTIALS-NO-SUBSCRIPTIONS" && <NoSubscriptions/>}
            {stage === "CREDENTIALS-CHOOSE-SUBSCRIPTION" &&
            <form onSubmit={(e) => { e.preventDefault(); performAssessment(selectedSubscription.value)}}>
                <div className="email-label">Logged in as {azureUserEmail}</div>
                <FormFieldSelect
                    label="Select your Microsoft Azure Subscription"
                    name="subscription-selector"
                    value={selectedSubscription}
                    placeholder="Select a subscription"
                    onChange={selectedSubscription => selectSubscription(selectedSubscription)}
                    isClearable={false}
                    isSearchable={true}
                    options={azureSubscriptions}
                />

                <button className={styles["submit-btn"]} type="submit">Start Cloud Assessment</button>

                {errorMessage &&
                    <div className="error" ref={triggerError}>
                        {errorMessage}
                    </div>
                }
            </form>
            }
            <NeedHelpContactUsWidget/>
            {(stage === "CREDENTIALS-FETCH-SUBSCRIPTIONS-FAILED" || stage === "CREDENTIALS-NO-SUBSCRIPTIONS" || stage === "CREDENTIALS-CHOOSE-SUBSCRIPTION") && <Logout logout={logout}/>}
        </div>
    }

};

const mapStateToProps = state => {
    let stage = state.cloudAssessment.stage;
    if(state.cloudAssessment.azureAccessToken && state.cloudAssessment.azureSubscriptions){
        stage = state.cloudAssessment.azureSubscriptions.length === 0 ? "CREDENTIALS-NO-SUBSCRIPTIONS" : "CREDENTIALS-CHOOSE-SUBSCRIPTION";
    }
    return {
        azureLoginToken: state.cloudAssessment.azureLoginToken,
        azureAccessToken: state.cloudAssessment.azureAccessToken,
        azureSubscriptions: state.cloudAssessment.azureSubscriptions,
        azureUserEmail: state.cloudAssessment.azureUserEmail,
        stage,
        errorMessage: state.cloudAssessment.errorMessage,
        selectedSubscription: state.cloudAssessment.selectedSubscription
    };
};

const mapDispatchToProps = dispatch => {
    return {
        login: () => {
          dispatch({
              type: "CLOUD-ASSESSMENT:AZURE-LOGIN"
          })
        },
        logout: () => {
            dispatch({
                type: "CLOUD-ASSESSMENT:AZURE-LOGOUT"
            })
        },
        performAssessment: (selectedSubscription) => {
            if(!selectedSubscription){
                dispatch({
                    type: "CLOUD-ASSESSMENT:AZURE-ERROR-UI"
                })
            } else {
                dispatch({
                    type: "CLOUD-ASSESSMENT:AZURE-STARTED"
                })
            }
        },
        selectSubscription: (selectedSubscription) => {
            dispatch({
                type: "CLOUD-ASSESSMENT:AZURE-SELECT-SUBSCRIPTION",
                payload: { selectedSubscription }
            })
        }
    }
};

export default connect(mapStateToProps, mapDispatchToProps)(CredentialsStep);
