import React from 'react';
import connect from "react-redux/es/connect/connect";
import _ from 'lodash';
import { STATUS_SUCCESS } from "../../actions";
import styles from "./ConnectionManagementView.module.scss";
import buttonsStyles from "ui-components/src/button/button.module.scss";
import { FormFieldInput } from "ui-components";
import layout from "ui-components/src/layout/layout.module.scss";
import classNames from "classnames";
import RowCounter from "../widgets/RowCounter";
import SearchWidget from "../widgets/SearchWidget";
import TableWidgetRow from "../widgets/TableWidgetRow";

const mapStateToProps = state => {
    return {
        managementAxiosStatus: state.management.managementAxiosStatus,
        connections: state.management.connections,
        successMessage: state.management.successMessage,
        errorMessage: state.management.errorMessage,
    };
};

const mapDispatchToProps = dispatch => {
    return {
        fetchManagementAxios: () => {
            dispatch({
                type: "MANAGEMENT:FETCH-MANAGEMENT-TOKEN"
            })
        },
        fetchConnections: () => {
            dispatch({
                type: "MANAGEMENT:FETCH-CONNECTIONS"
            })
        },
        enableConnection: (connectionName, companyName, emailDomains) => {
            dispatch({
                type: "MANAGEMENT:ENABLE-CONNECTION",
                payload: {connectionName, companyName, emailDomains}
            })
        },
        disableConnection: (connectionName) => {
            dispatch({
                type: "MANAGEMENT:DISABLE-CONNECTION",
                payload: {connectionName}
            })
        },
        clearMessages: () => {
            dispatch({ type: 'MANAGEMENT:CLEAR-CONNECTIONS-MESSAGES' })
        }
    }
};

class ConnectionManagement extends React.Component {
    constructor(props) {
        super(props);
        this.state = {connectionName: '', companyName: '', emailDomains: ''};
        props.fetchManagementAxios();
        this.props.clearMessages();

        if (this.props.managementAxiosStatus === STATUS_SUCCESS) {
            this.props.fetchConnections();
        }

        // create a ref to store the textInput DOM element
        this.companyNameInput = React.createRef();

        this.handleChange = this.handleChange.bind(this);
        this.enableButtonClicked = this.enableButtonClicked.bind(this);
        this.disableButtonClicked = this.disableButtonClicked.bind(this);
    }

    componentDidUpdate(prevProps) {
        if (prevProps.managementAxiosStatus !== STATUS_SUCCESS && this.props.managementAxiosStatus === STATUS_SUCCESS) {
            this.props.fetchConnections();
        }
        if (!prevProps.connections && this.props.connections) {
            this.setState({ filteredConnections: this.props.connections });

        }
    }

    handleChange(event) {
        this.setState({[event.target.name]: event.target.value});
    }

    enableButtonClicked(event) {
        this.props.enableConnection(this.state.connectionName, this.state.companyName, this.state.emailDomains);
        event.preventDefault();
    }

    disableButtonClicked(event) {
        if (window.confirm(`Are you sure you want to disable Federation connection "${this.state.connectionName}"?`)) {
            this.props.disableConnection(this.state.connectionName);
            event.preventDefault();
        }
    }

    filterChanged(value) {
        this.setState({
            filteredConnections: _.filter(this.props.connections, ({ name, metadata, options }) => {
                const lowerCaseValue = value.toLowerCase();
                return _.includes(name.toLowerCase(), lowerCaseValue) ||
                    (metadata && metadata.company && _.includes(metadata.company.toLowerCase(), lowerCaseValue)) ||
                    (options && options.domain_aliases && _.find(options.domain_aliases, alias => _.includes(alias.toLowerCase(),  lowerCaseValue)));
            })
        });
    }

    connectionClicked({name, company, emailDomains}) {
        this.setState({
            connectionName: name,
            companyName: company || '',
            emailDomains: emailDomains ? emailDomains.join(',') : ''
        });
        this.props.clearMessages();
        this.companyNameInput.current.focus();
    }

    render() {
        return (
            <div className={classNames(styles["connection-management"], layout["main-content-container"])}>
                <div>
                    <h3>1. Select existing connections</h3>
                    {this.props.connections && <div>
                        <TableWidgetRow>
                            <RowCounter filteredCount={_.get(this.state, 'filteredConnections.length')} totalCount={this.props.connections.length}
                                        singularTitle="Connection" pluralTitle="Connections"/>
                            <SearchWidget setFilter={this.filterChanged.bind(this)}/>
                        </TableWidgetRow>
                        <table className={styles["connections-table"]}>
                            <tbody>
                            <tr>
                                <th>Connection Name</th>
                                <th>Company Name</th>
                                <th>Email Domains</th>
                                <th>Has Rule Configuration</th>
                            </tr>
                            {this.state.filteredConnections && this.state.filteredConnections.map(({ name, metadata, hasRuleConfig, id, options }) =>
                                <tr key={id}
                                    onClick={() => this.connectionClicked({
                                        name,
                                        company: metadata && metadata.company,
                                        emailDomains: options && options.domain_aliases
                                    })}>
                                    <td>{name}</td>
                                    <td>{metadata && metadata.company}</td>
                                    <td className={styles["multi-line-cell"]}>{options && options.domain_aliases ?
                                        options.domain_aliases.join('\n')
                                        : '-'}</td>
                                    <td>{hasRuleConfig ? '✓' : ''}</td>
                                </tr>
                            )}
                            </tbody>
                        </table>
                    </div>}
                    {!this.props.connections && <h4>Loading...</h4>}
                </div>
                <div>
                    <h3>2. Enable \ Disable Federation Connection</h3>
                    {this.props.managementAxiosStatus === STATUS_SUCCESS &&
                    <form onSubmit={this.handleSubmit} className={styles["connection-management-form"]}>
                        <label>
                            Connection Name
                            <p>{this.state.connectionName}</p>
                        </label>
                        <label>
                            Email Domains
                            <p>{this.state.emailDomains}</p>
                        </label>
                        <FormFieldInput
                            type="text"
                            name="companyName"
                            label="Company Name"
                            value={this.state.companyName}
                            reference={this.companyNameInput}
                            onChange={this.handleChange}
                        />
                        <div className={styles["button-pair"]}>
                            <button className={buttonsStyles["primary"]} onClick={this.enableButtonClicked}>Enable</button>
                            <button className={buttonsStyles["dangerous"]} onClick={this.disableButtonClicked}>Disable</button>
                        </div>
                        <div style={{color: 'red'}}>{this.props.errorMessage}</div>
                        <div style={{whiteSpace: 'pre-line'}}>{this.props.successMessage}</div>
                    </form>}
                    {this.props.managementAxiosStatus !== STATUS_SUCCESS && <h4>Loading...</h4>}
                </div>
            </div>
        );
    }
}

export default connect(mapStateToProps, mapDispatchToProps)(ConnectionManagement);
