import axios from 'axios';
import _ from 'lodash';
import links from "../../consts/links";
import { executeAllPromises } from "../../utils/utils";

let tenancyAxios;
let agentsAxios;

const getTenancyAxios = () => {
    if (tenancyAxios) return tenancyAxios;

    tenancyAxios = axios.create({
        baseURL: links.launchPad.services.tenancyBackend,
        headers: {
            'X-User-Token': axios.defaults.headers.common['Authorization']
        }
    });

    return tenancyAxios;
};

const getAgentsAxios = () => {
    if (agentsAxios) return agentsAxios;

    agentsAxios = axios.create({
        baseURL: links.launchPad.services.agentsManagementBackend,
        headers: {
            'X-User-Token': axios.defaults.headers.common['Authorization']
        }
    });

    return agentsAxios;
};

const getErrorMessage = (error, action) => {
    const serverErrorMessage = _.get(error, 'response.data.message') || _.get(error, 'response.data') || error.toString();
    return `Could not ${action}: ${serverErrorMessage}`;
};
const sendMessageToHostApplication = message => {
    window.top.postMessage(message, '*');
};
const closeWidget = () => {
    sendMessageToHostApplication({ type: 'closeAccountManagementWidget' });
};
const accountDataChanged = payload => {
    sendMessageToHostApplication({ type: 'accountDataChanged', payload });
};

export default ({ dispatch, next, action, getState }) => {
    const { type, payload } = action;
    const accountId = _.get(getState(), 'accountsManagement.accountId');

    if (type === "ACCOUNTS-MANAGEMENT:SET-TENANCY-BASE-URL") {
        getTenancyAxios().defaults.baseURL = payload.tenancyBaseUrl;
    } else if (type === "ACCOUNTS-MANAGEMENT:SET-AGENTS-BASE-URL") {
        getAgentsAxios().defaults.baseURL = payload.agentsBaseUrl;
    } else if (type === "ACCOUNTS-MANAGEMENT:SET-ACCOUNT-ID") {
        // Using timeout to make sure the accountId was updated in state before dispatching fetch-data:
        setTimeout(() => {
            dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA" });
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS") {
        getTenancyAxios().get(`/account`)
            .then(({ data: accounts }) => {
                const account = _.find(accounts, { accountPublicId: accountId });
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-SUCCESS",
                    payload: { accounts, account }
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-ERROR",
                    payload: { errorMessage: getErrorMessage(error, 'retrieve accounts') }
                });
            });
        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES") {
        Promise.all([
            getTenancyAxios().get(`/account/${accountId}/users`),
            getTenancyAxios().get(`/account/${accountId}/workspace`)
        ])
            .then(([{ data: users }, { data: workspaces }]) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-SUCCESS",
                    payload: { users, workspaces }
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-ERROR",
                    payload: { errorMessage: getErrorMessage(error, 'retrieve account users and workspaces') }
                });
            });
        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-AGENTS") {
        getAgentsAxios().get(`/agent?account_id=${accountId}`)
            .then(({ data: { agents } }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-SUCCESS",
                    payload: { agents }
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-ERROR",
                    payload: { errorMessage: getErrorMessage(error, 'retrieve account agents') }
                });
            });
        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-AGENTS-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS") {
        axios.get(`/subscriptions/${accountId}`)
            .then(({ data: subscriptions }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-SUCCESS",
                    payload: { subscriptions }
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-ERROR",
                    payload: { errorMessage: getErrorMessage(error, 'retrieve account subscriptions') }
                });
            });
        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-DATA") {
        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-ACCOUNTS" });
        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES" });
        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-AGENTS" });
        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS" });

        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER") {
        const { associatedUserPublicId } = payload;

        getTenancyAxios().post(`/account/${accountId}/disassociate`, { associatedUserPublicId })
            .then(() => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-SUCCESS",
                        payload: { associatedUserPublicId }
                    });
                    accountDataChanged({ action: 'disassociate-user', userId: associatedUserPublicId });
                },
                error => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-ERROR",
                        payload: { errorMessage: getErrorMessage(error, 'disassociate user') }
                    });
                });

        return next({ type: "ACCOUNTS-MANAGEMENT:DISASSOCIATE-USER-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES") {
        const { user, workspaceIdsToRemove, workspaceIdsToAdd } = payload;
        const addWorkspacesPromises = workspaceIdsToAdd.map(workspacePublicId => getTenancyAxios()
            .post(`/account/${accountId}/workspace/${workspacePublicId}/add-user/${user.userPublicId}`));
        const removeWorkspacesPromises = workspaceIdsToRemove.map(workspacePublicId => getTenancyAxios()
            .post(`/account/${accountId}/workspace/${workspacePublicId}/remove-user/${user.userPublicId}`));

        executeAllPromises(addWorkspacesPromises.concat(removeWorkspacesPromises))
            .then(() => {
                    dispatch({ type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-SUCCESS" });
                    accountDataChanged({
                        action: 'manage-user-workspaces',
                        userId: user.userPublicId,
                        addedWorkspaces: workspaceIdsToAdd,
                        removedWorkspaces: workspaceIdsToRemove
                    });
                },
                firstError => dispatch({
                    type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-ERROR",
                    payload: { errorMessage: getErrorMessage(firstError, "manage the user's workspaces") }
                }))
            .finally(() => dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES" }));

        return next({ type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME") {
        const { workspace, newName } = payload;

        // Why timeout? in case user makes a failed rename, we use prop failedRenameWorkspace to know which workspace
        // failed. In case the user tries _again_ to do failed workspace, we need the timeout to allow this prop to be
        // null before it updates again.
        setTimeout(() => {
            getTenancyAxios().patch(`/account/${accountId}/workspace/${workspace.workspacePublicId}`, { newName })
                .then(() => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-SUCCESS"
                    });
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES"
                    });
                    accountDataChanged({ action: 'rename-workspace', workspace: workspace.workspacePublicId, newName });
                }, error => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-ERROR",
                        payload: { workspace, errorMessage: getErrorMessage(error, `rename workspace ${workspace.workspaceName}`) }
                    });
                });
        });

        return next({
            type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-PENDING",
            payload: payload.workspace.workspacePublicId
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE") {
        const { workspacePublicId, workspaceName } = payload;

        getTenancyAxios().delete(`/account/${accountId}/workspace/${workspacePublicId}`)
            .then(() => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-SUCCESS",
                    payload
                });
                dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES" });
                accountDataChanged({ action: 'delete-workspace', workspace: workspacePublicId });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-ERROR",
                    payload: { errorMessage: getErrorMessage(error, `delete workspace ${workspaceName}`) }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-PENDING",
            payload: workspacePublicId
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE") {
        const { newName } = payload;

        getTenancyAxios().post(`/account/${accountId}/workspace/${newName}`)
            .then(({ data: newWorkspace }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-SUCCESS",
                    payload: newWorkspace
                });
                accountDataChanged({ action: 'add-new-workspace', newWorkspace });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-ERROR",
                    payload: { errorMessage: getErrorMessage(error, `add new workspace ${newName}`) }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL") {
        axios.get("/portal/does-user-exist", { params: { email: payload.userEmail } })
            .then(({ data }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-SUCCESS",
                    payload: { email: payload.userEmail, exists: data.doesExist }
                });
                if (data.doesExist) {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:SET-ADD-USER-OVERLAY-STATE",
                        payload: "role"
                    });
                }
            }, ({ response }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-ERROR",
                    payload: { errorMessage: `Could not check if user with email ${payload.userEmail} exists in Cloud Central: ${response.data.message}` }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER") {
        const { associatedEmail, rolePublicId, workspaces } = payload;

        const errorCallback = error => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-ERROR",
                payload: { errorMessage: getErrorMessage(error, `associate user`) }
            });
        };

        getTenancyAxios().post(`/account/${accountId}/associate`, { associatedEmail, rolePublicId })
            .then(({ data: user }) => {
                const associateUserToWorkspacesPromise = rolePublicId === "Role-2" ?
                    Promise.all(workspaces.map(workspace => getTenancyAxios()
                        .post(`/account/${accountId}/workspace/${workspace}/add-user/${user.userPublicId}`))) :
                    Promise.resolve();

                associateUserToWorkspacesPromise
                    .then(() => {
                        dispatch({
                            type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-SUCCESS",
                            payload: { user }
                        });
                        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES" });
                        accountDataChanged({ action: 'associate-user', associatedEmail, rolePublicId, workspaces });
                    }, errorCallback);
            }, errorCallback);

        return next({
            type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES") {
        const { agent, workspaceIdsToRemove, workspaceIdsToAdd } = payload;
        const addWorkspacesPromises = workspaceIdsToAdd.map(workspacePublicId => getTenancyAxios()
            .post(`/account/${accountId}/workspace/${workspacePublicId}/add-agent/${agent.agentId}`));
        const removeWorkspacesPromises = workspaceIdsToRemove.map(workspacePublicId => getTenancyAxios()
            .post(`/account/${accountId}/workspace/${workspacePublicId}/remove-agent/${agent.agentId}`));

        executeAllPromises(addWorkspacesPromises.concat(removeWorkspacesPromises))
            .then(() => {
                    dispatch({ type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-SUCCESS" });
                    accountDataChanged({
                        action: 'manage-agent-workspaces',
                        agentId: agent.agentId,
                        addedWorkspaces: workspaceIdsToAdd,
                        removedWorkspaces: workspaceIdsToRemove
                    });
                },
                firstError => dispatch({
                    type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-ERROR",
                    payload: { errorMessage: getErrorMessage(firstError, "manage the agent's workspaces") }
                }))
            .finally(() => dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-USERS-AND-WORKSPACES" }));

        return next({ type: "ACCOUNTS-MANAGEMENT:MANAGE-AGENT-WORKSPACES-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION") {
        const { subscription, newName } = payload;

        axios.patch(`/subscriptions/${accountId}/${subscription.subscriptionId}`, { subscriptionName: newName })
            .then(() => {
                dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS" });
                dispatch({ type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-SUCCESS" });
                accountDataChanged({
                    action: 'rename-subscription',
                    subscription: subscription.subscriptionId,
                    newName
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-ERROR",
                    payload: {
                        subscription,
                        errorMessage: getErrorMessage(error, `rename subscription ${subscription.subscriptionName}`)
                    }
                });
            });

        return next({ type: "ACCOUNTS-MANAGEMENT:RENAME-SUBSCRIPTION-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS") {
        const { subscription: { subscriptionId }, accountIdsToRemove, accountIdsToAdd } = payload;
        const addAccountsPromises = accountIdsToAdd.map(accountIdToAdd =>
            axios.post(`/subscriptions/${accountIdToAdd}`, { subscriptionId }));
        const removeAccountsPromises = accountIdsToRemove.map(accountIdToRemove =>
            axios.delete(`/subscriptions/${accountIdToRemove}/${subscriptionId}`));

        executeAllPromises(addAccountsPromises.concat(removeAccountsPromises))
            .then(() => {
                    dispatch({ type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-SUCCESS" });
                    accountDataChanged({
                        action: 'manage-subscription-account',
                        subscriptionId,
                        addedAccounts: accountIdsToAdd,
                        removedAccounts: accountIdsToRemove
                    });
                },
                firstError => dispatch({
                    type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-ERROR",
                    payload: { errorMessage: getErrorMessage(firstError, "manage the subscription's accounts") }
                }))
            .finally(() => dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-SUBSCRIPTIONS" }));

        return next({ type: "ACCOUNTS-MANAGEMENT:MANAGE-SUBSCRIPTION-ACCOUNTS-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:CLOSE-ACCOUNT-WIDGET") {
        dispatch({ type: "ACCOUNTS-MANAGEMENT:CLOSE-ALL-OVERLAYS" });
        closeWidget();
    }
    return next(action);
};
